@@ -32,7 +32,7 @@
 
 #include <telepathy-glib/interfaces.h>
 
-#define DEBUG_FLAG GABBLE_DEBUG_TUBES
+#define DEBUG_FLAG GABBLE_DEBUG_BYTESTREAM
 
 #include "bytestream-iface.h"
 #include "base64.h"
@@ -73,6 +73,7 @@ enum
   PROP_PEER_JID,
   PROP_PEER_RESOURCE,
   PROP_STATE,
+  PROP_PROTOCOL,
   PROP_BLOCK_SIZE,
   LAST_PROPERTY
 };
@@ -178,6 +179,9 @@ gabble_bytestream_ibb_get_property (GObject *object,
       case PROP_STATE:
         g_value_set_uint (value, priv->state);
         break;
+      case PROP_PROTOCOL:
+        g_value_set_string (value, NS_IBB);
+        break;
       case PROP_BLOCK_SIZE:
         g_value_set_uint (value, priv->block_size);
         break;
@@ -295,6 +299,8 @@ gabble_bytestream_ibb_class_init (
        "peer-jid");
    g_object_class_override_property (object_class, PROP_STATE,
        "state");
+   g_object_class_override_property (object_class, PROP_PROTOCOL,
+       "protocol");
 
   param_spec = g_param_spec_string (
       "peer-resource",
@@ -508,10 +514,14 @@ gabble_bytestream_ibb_receive (GabbleBytestreamIBB *self,
  * Implements gabble_bytestream_iface_accept on GabbleBytestreamIface
  */
 static void
-gabble_bytestream_ibb_accept (GabbleBytestreamIface *iface, LmMessage *msg)
+gabble_bytestream_ibb_accept (GabbleBytestreamIface *iface,
+                              GabbleBytestreamAugmentSiAcceptReply func,
+                              gpointer user_data)
 {
   GabbleBytestreamIBB *self = GABBLE_BYTESTREAM_IBB (iface);
   GabbleBytestreamIBBPrivate *priv = GABBLE_BYTESTREAM_IBB_GET_PRIVATE (self);
+  LmMessage *msg;
+  LmMessageNode *si;
 
   if (priv->state != GABBLE_BYTESTREAM_STATE_LOCAL_PENDING)
     {
@@ -519,10 +529,25 @@ gabble_bytestream_ibb_accept (GabbleBytestreamIface *iface, LmMessage *msg)
       return;
     }
 
+  msg = gabble_bytestream_factory_make_accept_iq (priv->peer_jid,
+      priv->stream_init_id, NS_IBB);
+  si = lm_message_node_get_child_with_namespace (msg->node, "si", NS_SI);
+  g_assert (si != NULL);
+
+  if (func != NULL)
+    {
+      /* let the caller add his profile specific data */
+      func (si, user_data);
+    }
+
   if (_gabble_connection_send (priv->conn, msg, NULL))
     {
+      DEBUG ("stream %s with %s is now accepted", priv->stream_id,
+          priv->peer_jid);
       g_object_set (self, "state", GABBLE_BYTESTREAM_STATE_ACCEPTED, NULL);
     }
+
+  lm_message_unref (msg);
 }
 
 static void
@@ -669,17 +694,6 @@ gabble_bytestream_ibb_initiate (GabbleBytestreamIface *iface)
   return TRUE;
 }
 
-/*
- * gabble_bytestream_ibb_get_protocol
- *
- * Implements gabble_bytestream_iface_get_protocol on GabbleBytestreamIface
- */
-static const gchar *
-gabble_bytestream_ibb_get_protocol (GabbleBytestreamIface *iface)
-{
-  return NS_IBB;
-}
-
 static void
 bytestream_iface_init (gpointer g_iface,
                        gpointer iface_data)
@@ -690,5 +704,4 @@ bytestream_iface_init (gpointer g_iface,
   klass->send = gabble_bytestream_ibb_send;
   klass->close = gabble_bytestream_ibb_close;
   klass->accept = gabble_bytestream_ibb_accept;
-  klass->get_protocol = gabble_bytestream_ibb_get_protocol;
 }
