@@ -55,21 +55,14 @@ gabble_bytestream_iface_close (GabbleBytestreamIface *self,
 
 void
 gabble_bytestream_iface_accept (GabbleBytestreamIface *self,
-                                LmMessage *msg)
+                                GabbleBytestreamAugmentSiAcceptReply func,
+                                gpointer user_data)
 {
-  void (*virtual_method)(GabbleBytestreamIface *, LmMessage *) =
+  void (*virtual_method)(GabbleBytestreamIface *,
+      GabbleBytestreamAugmentSiAcceptReply, gpointer) =
     GABBLE_BYTESTREAM_IFACE_GET_CLASS (self)->accept;
   g_assert (virtual_method != NULL);
-  virtual_method (self, msg);
-}
-
-const gchar *
-gabble_bytestream_iface_get_protocol (GabbleBytestreamIface *self)
-{
-  const gchar * (*virtual_method)(GabbleBytestreamIface *) =
-    GABBLE_BYTESTREAM_IFACE_GET_CLASS (self)->get_protocol;
-  g_assert (virtual_method != NULL);
-  return virtual_method (self);
+  virtual_method (self, func, user_data);
 }
 
 static void
@@ -152,6 +145,17 @@ gabble_bytestream_iface_base_init (gpointer klass)
           G_PARAM_STATIC_BLURB);
       g_object_interface_install_property (klass, param_spec);
 
+      param_spec = g_param_spec_string (
+          "protocol",
+          "protocol",
+          "the name of the protocol implemented by this bytestream",
+          NULL,
+          G_PARAM_READABLE |
+          G_PARAM_STATIC_NAME |
+          G_PARAM_STATIC_NICK |
+          G_PARAM_STATIC_BLURB);
+      g_object_interface_install_property (klass, param_spec);
+
       initialized = TRUE;
     }
 }
