@@ -34,14 +34,6 @@ static const Feature self_advertised_features[] =
   { VERSION, NS_CHAT_STATES, PRESENCE_CAP_CHAT_STATES},
   { VERSION, NS_NICK, 0},
   { VERSION, NS_NICK "+notify", 0},
-  { VERSION, NS_OLPC_BUDDY_PROPS, 0},
-  { VERSION, NS_OLPC_BUDDY_PROPS "+notify", 0},
-  { VERSION, NS_OLPC_ACTIVITIES, 0},
-  { VERSION, NS_OLPC_ACTIVITIES "+notify", 0},
-  { VERSION, NS_OLPC_CURRENT_ACTIVITY, 0},
-  { VERSION, NS_OLPC_CURRENT_ACTIVITY "+notify", 0},
-  { VERSION, NS_OLPC_ACTIVITY_PROPS, 0},
-  { VERSION, NS_OLPC_ACTIVITY_PROPS "+notify", 0},
   { VERSION, NS_SI, PRESENCE_CAP_SI},
   { VERSION, NS_IBB, PRESENCE_CAP_IBB},
   { VERSION, NS_TUBES, PRESENCE_CAP_SI_TUBES},
@@ -51,6 +43,12 @@ static const Feature self_advertised_features[] =
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO},
   { BUNDLE_JINGLE_VIDEO, NS_JINGLE_DESCRIPTION_VIDEO,
     PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO},
+
+  { BUNDLE_OLPC_1, NS_OLPC_BUDDY_PROPS "+notify", PRESENCE_CAP_OLPC_1},
+  { BUNDLE_OLPC_1, NS_OLPC_ACTIVITIES "+notify", PRESENCE_CAP_OLPC_1},
+  { BUNDLE_OLPC_1, NS_OLPC_CURRENT_ACTIVITY "+notify", PRESENCE_CAP_OLPC_1},
+  { BUNDLE_OLPC_1, NS_OLPC_ACTIVITY_PROPS "+notify", PRESENCE_CAP_OLPC_1},
+
   { NULL, NULL, 0}
 };
 
