@@ -284,6 +284,8 @@ _got_self_avatar_for_get_known_avatar_tokens (GObject *obj,
 
   g_signal_handler_disconnect (obj, context->signal_conn);
 
+  g_assert (base->self_handle != 0);
+
   g_hash_table_insert (context->ret, GUINT_TO_POINTER (base->self_handle),
       g_strdup (sha1));
 
@@ -296,9 +298,9 @@ _got_self_avatar_for_get_known_avatar_tokens (GObject *obj,
 
 
 /**
- * gabble_connection_get_avatar_tokens
+ * gabble_connection_get_known_avatar_tokens
  *
- * Implements D-Bus method GetAvatarTokens
+ * Implements D-Bus method GetKnownAvatarTokens
  * on interface org.freedesktop.Telepathy.Connection.Interface.Avatars
  *
  * @context: The D-Bus invocation context to use to return values
@@ -613,6 +615,13 @@ emit_avatar_retrieved (TpSvcConnectionInterfaceAvatars *iface,
   g_string_free (avatar_str, TRUE);
 }
 
+/* All references are borrowed */
+typedef struct {
+    TpHandle handle;
+    GabbleConnection *conn;
+    TpSvcConnectionInterfaceAvatars *iface;
+} RequestAvatarsContext;
+
 static void
 request_avatars_cb (GabbleVCardManager *manager,
                     GabbleVCardManagerRequest *request,
@@ -621,10 +630,18 @@ request_avatars_cb (GabbleVCardManager *manager,
                     GError *vcard_error,
                     gpointer user_data)
 {
-  if (vcard_error != NULL)
-    return;
+  RequestAvatarsContext *ctx = user_data;
 
-  emit_avatar_retrieved (user_data, handle, vcard);
+  g_assert (g_hash_table_lookup (ctx->conn->avatar_requests,
+      GUINT_TO_POINTER (ctx->handle)));
+
+  g_hash_table_remove (ctx->conn->avatar_requests,
+      GUINT_TO_POINTER (ctx->handle));
+
+  if (vcard_error == NULL)
+    emit_avatar_retrieved (ctx->iface, handle, vcard);
+
+  g_slice_free (RequestAvatarsContext, ctx);
 }
 
 static void
@@ -658,8 +675,21 @@ gabble_connection_request_avatars (TpSvcConnectionInterfaceAvatars *iface,
         }
       else
         {
-          gabble_vcard_manager_request (self->vcard_manager,
-              contact, 0, request_avatars_cb, iface, NULL);
+          if (NULL == g_hash_table_lookup (self->avatar_requests,
+                GUINT_TO_POINTER (contact)))
+            {
+              RequestAvatarsContext *ctx = g_slice_new (RequestAvatarsContext);
+
+              ctx->conn = self;
+              ctx->iface = iface;
+              ctx->handle = contact;
+
+              g_hash_table_insert (self->avatar_requests,
+                  GUINT_TO_POINTER (contact), ctx);
+
+              gabble_vcard_manager_request (self->vcard_manager,
+                contact, 0, request_avatars_cb, ctx, NULL);
+            }
         }
     }
 
@@ -783,7 +813,7 @@ gabble_connection_set_avatar (TpSvcConnectionInterfaceAvatars *iface,
  * @context: The D-Bus invocation context to use to return values
  *           or throw an error.
  */
-void
+static void
 gabble_connection_clear_avatar (TpSvcConnectionInterfaceAvatars *iface,
                                 DBusGMethodInvocation *context)
 {
