@@ -23,6 +23,7 @@
 #include <string.h>
 
 #include <telepathy-glib/channel-iface.h>
+#include <telepathy-glib/util.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_OLPC
 
@@ -434,7 +435,7 @@ transmit_properties (GabbleConnection *conn,
 }
 
 static GQuark
-preload_buddy_properties_quark ()
+preload_buddy_properties_quark (void)
 {
   static GQuark q = 0;
   if (q == 0)
@@ -446,7 +447,7 @@ preload_buddy_properties_quark ()
 }
 
 static GQuark
-invitees_quark ()
+invitees_quark (void)
 {
   static GQuark q = 0;
   if (q == 0)
@@ -671,8 +672,6 @@ extract_activities (GabbleConnection *conn,
       const gchar *act_id;
       const gchar *room;
       ActivityInfo *info;
-      TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
-          (TpBaseConnection *) conn, TP_HANDLE_TYPE_ROOM);
       TpHandle room_handle;
 
       if (tp_strdiff (node->name, "activity"))
@@ -1027,7 +1026,7 @@ olpc_buddy_info_set_activities (GabbleSvcOLPCBuddyInfo *iface,
         {
           if (tp_handle_set_is_member (activities_set, channel))
             {
-              GError *error = g_error_new (TP_ERRORS,
+              error = g_error_new (TP_ERRORS,
                   TP_ERROR_INVALID_ARGUMENT,
                   "Can't set twice the same activity: %s", room);
 
