@@ -21,9 +21,15 @@
 #ifndef __CONN_PRESENCE_H__
 #define __CONN_PRESENCE_H__
 
+#include <glib.h>
+
+#include "gabble-connection.h"
+
 G_BEGIN_DECLS
 
+void conn_presence_class_init (GabbleConnectionClass *klass);
 void conn_presence_init (GabbleConnection *conn);
+void conn_presence_finalize (GabbleConnection *conn);
 void conn_presence_iface_init (gpointer g_iface, gpointer iface_data);
 
 G_END_DECLS
