@@ -24,7 +24,7 @@ gabble_debug_set_log_file_from_env (void)
   if (output_file == NULL)
     return;
 
-  out = g_open (output_file, O_WRONLY | O_CREAT, 0644);
+  out = g_open (output_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
   if (out == -1)
     {
       g_warning ("Can't open logfile '%s': %s", output_file,
@@ -80,13 +80,9 @@ void gabble_debug_set_flags_from_env ()
 
   flags_string = g_getenv ("GABBLE_DEBUG");
 
-#ifdef HAVE_TP_DEBUG_SET_FLAGS
-      tp_debug_set_flags (flags_string);
-#else
-      tp_debug_set_flags_from_string (flags_string);
-#endif
+  tp_debug_set_flags (flags_string);
 
-  if (flags_string)
+  if (flags_string != NULL)
     {
       gabble_debug_set_flags (g_parse_debug_string (flags_string, keys,
             nkeys));
