@@ -36,7 +36,7 @@ typedef enum
   GABBLE_DEBUG_BYTESTREAM    = 1 << 16,
 } GabbleDebugFlags;
 
-void gabble_debug_set_flags_from_env ();
+void gabble_debug_set_flags_from_env (void);
 void gabble_debug_set_flags (GabbleDebugFlags flags);
 gboolean gabble_debug_flag_is_set (GabbleDebugFlags flag);
 void gabble_debug (GabbleDebugFlags flag, const gchar *format, ...)
