@@ -82,7 +82,7 @@ gabble_disco_error_quark (void)
   return quark;
 }
 
-#define GABBLE_DISCO_GET_PRIVATE(o)     ((GabbleDiscoPrivate*)((o)->priv));
+#define GABBLE_DISCO_GET_PRIVATE(o)     ((GabbleDiscoPrivate*)((o)->priv))
 
 static void
 gabble_disco_init (GabbleDisco *obj)
@@ -190,14 +190,12 @@ gabble_disco_constructor (GType type, guint n_props,
 
 static void cancel_request (GabbleDiscoRequest *request);
 
-void
+static void
 gabble_disco_dispose (GObject *object)
 {
   GabbleDisco *self = GABBLE_DISCO (object);
   GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (self);
   GSList *l;
-  DBusGProxy *bus_proxy;
-  bus_proxy = tp_get_bus_proxy ();
 
   if (priv->dispose_has_run)
     return;
@@ -228,7 +226,7 @@ gabble_disco_dispose (GObject *object)
     G_OBJECT_CLASS (gabble_disco_parent_class)->dispose (object);
 }
 
-void
+static void
 gabble_disco_finalize (GObject *object)
 {
   DEBUG ("called with %p", object);
@@ -295,6 +293,7 @@ static gboolean
 timeout_request (gpointer data)
 {
   GabbleDiscoRequest *request = (GabbleDiscoRequest*) data;
+  GabbleDisco *disco;
   GError *err /* doesn't need initializing */;
   g_return_val_if_fail (data != NULL, FALSE);
 
@@ -302,12 +301,29 @@ timeout_request (gpointer data)
       "Request for %s on %s timed out",
       (request->type == GABBLE_DISCO_TYPE_INFO)?"info":"items",
       request->jid);
+
+  /* Temporarily ref the disco object to avoid crashing if the callback
+   * destroys us (as seen in test-disco-no-reply.py) */
+  disco = g_object_ref (request->disco);
+
+  /* also, we're about to run the callback, so it's too late to cancel it -
+   * avoid crashing if running the callback destroys the bound object */
+  if (NULL != request->bound_object)
+    {
+      g_object_weak_unref (request->bound_object, notify_delete_request,
+          request);
+      request->bound_object = NULL;
+    }
+
   (request->callback)(request->disco, request, request->jid, request->node,
                       NULL, err, request->user_data);
   g_error_free (err);
 
   request->timer_id = 0;
   delete_request (request);
+
+  g_object_unref (disco);
+
   return FALSE;
 }
 
@@ -861,13 +877,14 @@ gabble_disco_conn_status_changed_cb (GabbleConnection *conn,
   if (status == TP_CONNECTION_STATUS_CONNECTED)
     {
       char *server;
+      gpointer pipeline;
 
       g_object_get (priv->connection, "stream-server", &server, NULL);
 
       g_assert (server != NULL);
 
       DEBUG ("connected, initiating service discovery on %s", server);
-      gpointer pipeline = gabble_disco_pipeline_init (disco, services_cb,
+      pipeline = gabble_disco_pipeline_init (disco, services_cb,
           end_cb, disco);
       gabble_disco_pipeline_run (pipeline, server);
 
