@@ -68,6 +68,8 @@
 #include "util.h"
 #include "vcard-manager.h"
 
+static guint disco_reply_timeout = 5000;
+
 #define GABBLE_TP_CAPABILITY_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
       G_TYPE_STRING, G_TYPE_UINT, G_TYPE_INVALID))
 #define GABBLE_TP_CAPABILITIES_CHANGED_MONSTER_TYPE (dbus_g_type_get_struct \
@@ -245,6 +247,8 @@ gabble_connection_constructor (GType type,
 
   self->bytestream_factory = gabble_bytestream_factory_new (self);
 
+  self->avatar_requests = g_hash_table_new (NULL, NULL);
+
   return (GObject *)self;
 }
 
@@ -726,6 +730,8 @@ gabble_connection_dispose (GObject *object)
   g_hash_table_destroy (self->olpc_pep_activities);
   g_hash_table_destroy (self->olpc_invited_activities);
 
+  g_hash_table_destroy (self->avatar_requests);
+
   /* if this is not already the case, we'll crash anyway */
   g_assert (!lm_connection_is_open (self->lmconn));
 
@@ -1150,18 +1156,10 @@ _gabble_connection_connect (TpBaseConnection *base,
       if (priv->port != 0)
         lm_connection_set_port (conn->lmconn, priv->port);
     }
-#ifndef HAVE_LM_SRV_LOOKUPS
-  /* set the server from the JID if we don't have SRV lookups */
   else
     {
-      DEBUG ("SRV lookup not supported, will connect to %s",
-          priv->stream_server);
-
-      lm_connection_set_server (conn->lmconn, priv->stream_server);
+      DEBUG ("letting SRV lookup decide server and port");
     }
-#else
-  DEBUG ("letting SRV lookup decide server and port");
-#endif /* HAVE_LM_SRV_LOOKUPS */
 
   if (priv->https_proxy_server)
     {
@@ -1181,7 +1179,6 @@ _gabble_connection_connect (TpBaseConnection *base,
       lm_connection_set_ssl (conn->lmconn, ssl);
       lm_ssl_unref (ssl);
     }
-#ifdef HAVE_LM_STARTTLS
   else
     {
       LmSSL *ssl = lm_ssl_new (NULL, connection_ssl_cb, conn, NULL);
@@ -1194,7 +1191,6 @@ _gabble_connection_connect (TpBaseConnection *base,
       if (!priv->require_encryption)
           priv->ignore_ssl_errors = TRUE;
     }
-#endif /* HAVE_LM_STARTTLS */
 
   /* send whitespace to the server every 30 seconds */
   lm_connection_set_keep_alive_rate (conn->lmconn, 30);
@@ -1511,9 +1507,10 @@ connection_iq_disco_cb (LmMessageHandler *handler,
 
       if (NULL == node || !tp_strdiff (suffix, feature->bundle))
         {
-          LmMessageNode *node = lm_message_node_add_child (result_query,
-              "feature", NULL);
-          lm_message_node_set_attribute (node, "var", feature->ns);
+          LmMessageNode *feature_node = lm_message_node_add_child
+              (result_query, "feature", NULL);
+
+          lm_message_node_set_attribute (feature_node, "var", feature->ns);
         }
     }
 
@@ -1846,11 +1843,7 @@ connection_auth_cb (LmConnection *lmconn,
     }
 
 
-#ifdef HAVE_LM_CONNECTION_GET_FULL_JID
   jid = lm_connection_get_full_jid (lmconn);
-#else
-  jid = lm_connection_get_jid (lmconn);
-#endif /* HAVE_LM_CONNECTION_GET_FULL_JID */
 
   base->self_handle = tp_handle_ensure (contact_handles, jid, NULL, &error);
 
@@ -1893,7 +1886,8 @@ connection_auth_cb (LmConnection *lmconn,
       capabilities_get_initial_caps (), priv->caps_serial++);
 
   if (!gabble_disco_request_with_timeout (conn->disco, GABBLE_DISCO_TYPE_INFO,
-                                          priv->stream_server, NULL, 5000,
+                                          priv->stream_server, NULL,
+                                          disco_reply_timeout,
                                           connection_disco_cb, conn,
                                           G_OBJECT (conn), &error))
     {
@@ -1942,6 +1936,11 @@ connection_disco_cb (GabbleDisco *disco,
   if (disco_error)
     {
       DEBUG ("got disco error, setting no features: %s", disco_error->message);
+      if (disco_error->code == GABBLE_DISCO_ERROR_TIMEOUT)
+        {
+          DEBUG ("didn't receive a response to our disco request: disconnect");
+          goto ERROR;
+        }
     }
   else
     {
@@ -2006,7 +2005,7 @@ connection_disco_cb (GabbleDisco *disco,
   return;
 
 ERROR:
-  if (error)
+  if (error != NULL)
     g_error_free (error);
 
   tp_base_connection_change_status (base,
@@ -2042,16 +2041,16 @@ _emit_capabilities_changed (GabbleConnection *conn,
       if (ccd->c2tf_fn (old_caps | new_caps))
         {
           GValue caps_monster_struct = {0, };
-          guint old_tpflags = ccd->c2tf_fn (old_caps);
-          guint old_caps = old_tpflags ?
+          guint old_specific = ccd->c2tf_fn (old_caps);
+          guint old_generic = old_specific ?
             TP_CONNECTION_CAPABILITY_FLAG_CREATE |
             TP_CONNECTION_CAPABILITY_FLAG_INVITE : 0;
-          guint new_tpflags = ccd->c2tf_fn (new_caps);
-          guint new_caps = new_tpflags ?
+          guint new_specific = ccd->c2tf_fn (new_caps);
+          guint new_generic = new_specific ?
             TP_CONNECTION_CAPABILITY_FLAG_CREATE |
             TP_CONNECTION_CAPABILITY_FLAG_INVITE : 0;
 
-          if (0 == (old_tpflags ^ new_tpflags))
+          if (0 == (old_specific ^ new_specific))
             continue;
 
           g_value_init (&caps_monster_struct,
@@ -2063,10 +2062,10 @@ _emit_capabilities_changed (GabbleConnection *conn,
           dbus_g_type_struct_set (&caps_monster_struct,
               0, handle,
               1, ccd->iface,
-              2, old_caps,
-              3, new_caps,
-              4, old_tpflags,
-              5, new_tpflags,
+              2, old_generic,
+              3, new_generic,
+              4, old_specific,
+              5, new_specific,
               G_MAXUINT);
 
           g_ptr_array_add (caps_arr, g_value_get_boxed (&caps_monster_struct));
@@ -2113,7 +2112,7 @@ connection_capabilities_update_cb (GabblePresenceCache *cache,
 static void
 gabble_connection_advertise_capabilities (TpSvcConnectionInterfaceCapabilities *iface,
                                           const GPtrArray *add,
-                                          const gchar **remove,
+                                          const gchar **del,
                                           DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
@@ -2133,28 +2132,28 @@ gabble_connection_advertise_capabilities (TpSvcConnectionInterfaceCapabilities *
   for (i = 0; i < add->len; i++)
     {
       GValue iface_flags_pair = {0, };
-      gchar *iface;
+      gchar *channel_type;
       guint flags;
 
       g_value_init (&iface_flags_pair, GABBLE_TP_CAPABILITY_PAIR_TYPE);
       g_value_set_static_boxed (&iface_flags_pair, g_ptr_array_index (add, i));
 
       dbus_g_type_struct_get (&iface_flags_pair,
-                              0, &iface,
+                              0, &channel_type,
                               1, &flags,
                               G_MAXUINT);
 
       for (ccd = capabilities_conversions; NULL != ccd->iface; ccd++)
-          if (g_str_equal (iface, ccd->iface))
+          if (g_str_equal (channel_type, ccd->iface))
             add_caps |= ccd->tf2c_fn (flags);
 
-      g_free (iface);
+      g_free (channel_type);
     }
 
-  for (i = 0; NULL != remove[i]; i++)
+  for (i = 0; NULL != del[i]; i++)
     {
       for (ccd = capabilities_conversions; NULL != ccd->iface; ccd++)
-          if (g_str_equal (remove[i], ccd->iface))
+          if (g_str_equal (del[i], ccd->iface))
             remove_caps |= ccd->tf2c_fn (~0);
     }
 
@@ -2251,6 +2250,7 @@ gabble_connection_get_capabilities (TpSvcConnectionInterfaceCapabilities *iface,
     {
       dbus_g_method_return_error (context, error);
       g_error_free (error);
+      return;
     }
 
   ret = g_ptr_array_new ();
@@ -2627,7 +2627,7 @@ room_jid_disco_cb (GabbleDisco *disco,
  */
 static gboolean
 room_jid_verify (RoomVerifyBatch *batch,
-                 guint index,
+                 guint i,
                  DBusGMethodInvocation *context)
 {
   gchar *room, *service;
@@ -2635,13 +2635,13 @@ room_jid_verify (RoomVerifyBatch *batch,
   GError *error = NULL;
 
   room = service = NULL;
-  gabble_decode_jid (batch->contexts[index].jid, &room, &service, NULL);
+  gabble_decode_jid (batch->contexts[i].jid, &room, &service, NULL);
 
   if (room == NULL || *room == '\0' || service == NULL || *service == '\0')
     {
       g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "unable to get room name and service from JID %s",
-          batch->contexts[index].jid);
+          batch->contexts[i].jid);
 
       ret = FALSE;
 
@@ -2650,7 +2650,7 @@ room_jid_verify (RoomVerifyBatch *batch,
 
   ret = (gabble_disco_request (batch->conn->disco, GABBLE_DISCO_TYPE_INFO,
                                service, NULL, room_jid_disco_cb,
-                               batch->contexts + index,
+                               batch->contexts + i,
                                G_OBJECT (batch->conn), &error) != NULL);
 
 out:
@@ -2781,3 +2781,10 @@ capabilities_service_iface_init (gpointer g_iface, gpointer iface_data)
 #undef IMPLEMENT
 }
 
+
+/* For unit tests only */
+void
+gabble_connection_set_disco_reply_timeout (guint timeout)
+{
+  disco_reply_timeout = timeout;
+}
