@@ -1,8 +1,7 @@
 /*
  * gabble-error.c - Source for Gabble's error handling API
- * Copyright (C) 2006 Collabora Ltd.
+ * Copyright (C) 2006-2007 Collabora Ltd.
  * Copyright (C) 2006 Nokia Corporation
- *   @author Ole Andre Vadla Ravnaas <ole.andre.ravnaas@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,11 +18,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-error.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "gabble-error.h"
 #include "namespaces.h"
+#include "util.h"
 
 #define MAX_LEGACY_ERRORS 3
 
@@ -38,13 +39,22 @@ typedef struct {
 
 static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
 {
+    {
+      "undefined-condition",
+      "application-specific condition",
+      NULL,
+      0,
+      NS_XMPP_STANZAS,
+      { 500, 0, },
+    },
+
     {
       "redirect",
       "the recipient or server is redirecting requests for this information "
       "to another entity",
       "modify",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 302, 0, },
     },
 
@@ -53,7 +63,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the recipient or server can no longer be contacted at this address",
       "modify",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 302, 0, },
     },
 
@@ -62,7 +72,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the sender has sent XML that is malformed or that cannot be processed",
       "modify",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 400, 0, },
     },
     {
@@ -71,7 +81,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "it at this time",
       "wait",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 400, 0, },
     },
     {
@@ -81,7 +91,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "to the syntax defined in Addressing Scheme (Section 3)",
       "modify",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 400, 0, },
     },
 
@@ -91,7 +101,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "perform the action, or has provided improper credentials",
       "auth",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 401, 0, },
     },
 
@@ -101,7 +111,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "service because payment is required",
       "auth",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 402, 0, },
     },
 
@@ -111,7 +121,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "perform the action",
       "auth",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 403, 0, },
     },
 
@@ -120,7 +130,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the addressed JID or item requested cannot be found",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 404, 0, },
     },
     {
@@ -128,7 +138,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the intended recipient is temporarily unavailable",
       "wait",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 404, 0, },
     },
     {
@@ -138,7 +148,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "contacted within a reasonable amount of time",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 404, 0, },
     },
 
@@ -147,7 +157,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the recipient or server does not allow any entity to perform the action",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 405, 0, },
     },
 
@@ -158,7 +168,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "or server (e.g., a local policy regarding acceptable words in messages)",
       "modify",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 406, 0, },
     },
 
@@ -168,7 +178,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "because registration is required",
       "auth",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 407, 0, },
     },
     {
@@ -177,7 +187,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "because a subscription is required",
       "auth",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 407, 0, },
     },
 
@@ -188,7 +198,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "contacted within a reasonable amount of time",
       "wait",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 408, 504, 0, },
     },
 
@@ -198,7 +208,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "with the same name or address",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 409, 0, },
     },
 
@@ -208,15 +218,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "or an otherwise-undefined internal server error",
       "wait",
       0,
-      NULL,
-      { 500, 0, },
-    },
-    {
-      "undefined-condition",
-      "application-specific condition",
-      NULL,
-      0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 500, 0, },
     },
     {
@@ -225,7 +227,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "the request",
       "wait",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 500, 0, },
     },
 
@@ -235,7 +237,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "therefore cannot be processed",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 501, 0, },
     },
 
@@ -245,7 +247,7 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       "service",
       "cancel",
       0,
-      NULL,
+      NS_XMPP_STANZAS,
       { 502, 503, 510, },
     },
 
@@ -287,6 +289,24 @@ static const XmppErrorSpec xmpp_errors[NUM_XMPP_ERRORS] =
       NS_JINGLE_ERRORS,
       { 0, },
     },
+
+    {
+      "no-valid-streams",
+      "None of the available streams are acceptable.",
+      "cancel",
+      XMPP_ERROR_BAD_REQUEST,
+      NS_SI,
+      { 400, 0 },
+    },
+
+    {
+      "bad-profile",
+      "The profile is not understood or invalid.",
+      "modify",
+      XMPP_ERROR_BAD_REQUEST,
+      NS_SI,
+      { 400, 0 },
+    },
 };
 
 GQuark
@@ -304,12 +324,17 @@ gabble_xmpp_error_from_node (LmMessageNode *error_node)
   gint i, j;
   const gchar *error_code_str;
 
+  g_return_val_if_fail (error_node != NULL, XMPP_ERROR_UNDEFINED_CONDITION);
+
   /* First, try to look it up the modern way */
   if (error_node->children)
     {
-      for (i = 0; i < NUM_XMPP_ERRORS; i++)
+      /* we loop backwards because the most specific errors are the larger
+       * numbers; the >= 0 test is OK because i is signed */
+      for (i = NUM_XMPP_ERRORS - 1; i >= 0; i--)
         {
-          if (lm_message_node_get_child (error_node, xmpp_errors[i].name))
+          if (lm_message_node_get_child_with_namespace (error_node,
+                xmpp_errors[i].name, xmpp_errors[i].namespace))
             {
               return i;
             }
@@ -324,7 +349,9 @@ gabble_xmpp_error_from_node (LmMessageNode *error_node)
 
       error_code = atoi (error_code_str);
 
-      for (i = 0; i < NUM_XMPP_ERRORS; i++)
+      /* skip UNDEFINED_CONDITION, we want code 500 to be translated
+       * to INTERNAL_SERVER_ERROR */
+      for (i = 1; i < NUM_XMPP_ERRORS; i++)
         {
           const XmppErrorSpec *spec = &xmpp_errors[i];
 
@@ -340,14 +367,15 @@ gabble_xmpp_error_from_node (LmMessageNode *error_node)
         }
     }
 
-  return INVALID_XMPP_ERROR;
+  return XMPP_ERROR_UNDEFINED_CONDITION;
 }
 
-GError *
+static GError *
 gabble_xmpp_error_to_g_error (GabbleXmppError error)
 {
   if (error >= NUM_XMPP_ERRORS)
-    return NULL;
+      return g_error_new (GABBLE_XMPP_ERROR, XMPP_ERROR_UNDEFINED_CONDITION,
+          "Unknown or invalid XMPP error");
 
   return g_error_new (GABBLE_XMPP_ERROR,
                       error,
@@ -366,8 +394,8 @@ gabble_xmpp_error_to_node (GabbleXmppError error,
   LmMessageNode *error_node, *node;
   gchar str[6];
 
-  if (error >= NUM_XMPP_ERRORS)
-    return NULL;
+  g_return_val_if_fail (error != XMPP_ERROR_UNDEFINED_CONDITION &&
+      error < NUM_XMPP_ERRORS, NULL);
 
   if (xmpp_errors[error].specialises)
     {
@@ -423,3 +451,28 @@ gabble_xmpp_error_description (GabbleXmppError error)
     return NULL;
 }
 
+GError *
+gabble_message_get_xmpp_error (LmMessage *msg)
+{
+  g_return_val_if_fail (msg != NULL, NULL);
+
+  if (lm_message_get_sub_type (msg) == LM_MESSAGE_SUB_TYPE_ERROR)
+    {
+      LmMessageNode *error_node = lm_message_node_get_child (msg->node,
+          "error");
+
+      if (error_node != NULL)
+        {
+          return gabble_xmpp_error_to_g_error
+              (gabble_xmpp_error_from_node (error_node));
+        }
+      else
+        {
+          return g_error_new (GABBLE_XMPP_ERROR,
+              XMPP_ERROR_UNDEFINED_CONDITION, "Unknown or invalid XMPP error");
+        }
+    }
+
+  /* no error */
+  return NULL;
+}
