@@ -25,7 +25,7 @@
 #include <time.h>
 
 #include <telepathy-glib/enums.h>
-#include <telepathy-glib/text-mixin.h>
+#include "text-mixin.h"
 
 G_BEGIN_DECLS
 
@@ -35,34 +35,38 @@ typedef struct _GabbleIMChannelClass GabbleIMChannelClass;
 struct _GabbleIMChannelClass {
     GObjectClass parent_class;
 
-    TpTextMixinClass text_class;
+    GabbleTextMixinClass text_class;
 };
 
 struct _GabbleIMChannel {
     GObject parent;
 
-    TpTextMixin text;
+    GabbleTextMixin text;
 
     gpointer priv;
 };
 
-GType gabble_im_channel_get_type(void);
+GType gabble_im_channel_get_type (void);
 
 /* TYPE MACROS */
 #define GABBLE_TYPE_IM_CHANNEL \
-  (gabble_im_channel_get_type())
+  (gabble_im_channel_get_type ())
 #define GABBLE_IM_CHANNEL(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj), GABBLE_TYPE_IM_CHANNEL, GabbleIMChannel))
 #define GABBLE_IM_CHANNEL_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass), GABBLE_TYPE_IM_CHANNEL, GabbleIMChannelClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass), GABBLE_TYPE_IM_CHANNEL,\
+                           GabbleIMChannelClass))
 #define GABBLE_IS_IM_CHANNEL(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj), GABBLE_TYPE_IM_CHANNEL))
 #define GABBLE_IS_IM_CHANNEL_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass), GABBLE_TYPE_IM_CHANNEL))
 #define GABBLE_IM_CHANNEL_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_IM_CHANNEL, GabbleIMChannelClass))
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_IM_CHANNEL, \
+                              GabbleIMChannelClass))
 
-gboolean _gabble_im_channel_receive (GabbleIMChannel *chan, TpChannelTextMessageType type, TpHandle sender, const char *from, time_t timestamp, const char *text);
+void _gabble_im_channel_receive (GabbleIMChannel *chan,
+    TpChannelTextMessageType type, TpHandle sender, const char *from,
+    time_t timestamp, const char *text);
 void _gabble_im_channel_state_receive (GabbleIMChannel *chan, guint state);
 
 G_END_DECLS
