@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-media-channel.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,19 +40,17 @@
 #include <telepathy-glib/svc-channel.h>
 #include <telepathy-glib/svc-properties-interface.h>
 
-#include "gabble-media-channel.h"
-
 #include "gabble-media-session.h"
 #include "gabble-media-stream.h"
 
 #include "media-factory.h"
 
-#define TP_SESSION_HANDLER_SET_TYPE (dbus_g_type_get_struct ("GValueArray", \
+#define GABBLE_TP_SESSION_HANDLER_SET_TYPE (dbus_g_type_get_struct ("GValueArray", \
       DBUS_TYPE_G_OBJECT_PATH, \
       G_TYPE_STRING, \
       G_TYPE_INVALID))
 
-#define TP_CHANNEL_STREAM_TYPE (dbus_g_type_get_struct ("GValueArray", \
+#define GABBLE_TP_CHANNEL_STREAM_TYPE (dbus_g_type_get_struct ("GValueArray", \
       G_TYPE_UINT, \
       G_TYPE_UINT, \
       G_TYPE_UINT, \
@@ -59,15 +59,21 @@
       G_TYPE_UINT, \
       G_TYPE_INVALID))
 
+static void call_state_iface_init (gpointer, gpointer);
 static void channel_iface_init (gpointer, gpointer);
+static void hold_iface_init (gpointer, gpointer);
 static void media_signalling_iface_init (gpointer, gpointer);
 static void streamed_media_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleMediaChannel, gabble_media_channel,
     G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_CALL_STATE,
+      call_state_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
       tp_group_mixin_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_HOLD,
+      hold_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_MEDIA_SIGNALLING,
       media_signalling_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_STREAMED_MEDIA,
@@ -112,9 +118,6 @@ const TpPropertySignature channel_property_signatures[NUM_CHAN_PROPS] = {
       { "gtalk-p2p-relay-token",  G_TYPE_STRING }
 };
 
-/* private structure */
-typedef struct _GabbleMediaChannelPrivate GabbleMediaChannelPrivate;
-
 struct _GabbleMediaChannelPrivate
 {
   GabbleConnection *conn;
@@ -127,12 +130,14 @@ struct _GabbleMediaChannelPrivate
 
   guint next_stream_id;
 
-  gboolean closed;
-  gboolean dispose_has_run;
+  TpLocalHoldState hold_state;
+  TpLocalHoldStateReason hold_state_reason;
+
+  gboolean closed:1;
+  gboolean dispose_has_run:1;
 };
 
-#define GABBLE_MEDIA_CHANNEL_GET_PRIVATE(obj) \
-    ((GabbleMediaChannelPrivate *)obj->priv)
+#define GABBLE_MEDIA_CHANNEL_GET_PRIVATE(obj) (obj->priv)
 
 static void
 gabble_media_channel_init (GabbleMediaChannel *self)
@@ -158,41 +163,43 @@ gabble_media_channel_constructor (GType type, guint n_props,
   TpBaseConnection *conn;
   DBusGConnection *bus;
   TpIntSet *set;
+  TpHandleRepoIface *contact_handles;
 
   obj = G_OBJECT_CLASS (gabble_media_channel_parent_class)->
            constructor (type, n_props, props);
 
   priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (GABBLE_MEDIA_CHANNEL (obj));
   conn = (TpBaseConnection *)priv->conn;
+  contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
 
   /* register object on the bus */
   bus = tp_get_bus ();
   dbus_g_connection_register_g_object (bus, priv->object_path, obj);
 
-  tp_group_mixin_init ((TpSvcChannelInterfaceGroup *)obj,
-      G_STRUCT_OFFSET (GabbleMediaChannel, group),
-      conn->handles[TP_HANDLE_TYPE_CONTACT],
-      conn->self_handle);
+  tp_group_mixin_init (obj, G_STRUCT_OFFSET (GabbleMediaChannel, group),
+      contact_handles, conn->self_handle);
 
   /* automatically add creator to channel */
   set = tp_intset_new ();
   tp_intset_add (set, priv->creator);
 
-  tp_group_mixin_change_members ((TpSvcChannelInterfaceGroup *)obj,
-      "", set, NULL, NULL, NULL, 0, 0);
+  tp_group_mixin_change_members (obj, "", set, NULL, NULL, NULL, 0, 0);
 
   tp_intset_destroy (set);
 
   /* allow member adding */
-  tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
-      TP_CHANNEL_GROUP_FLAG_CAN_ADD, 0);
+  tp_group_mixin_change_flags (obj, TP_CHANNEL_GROUP_FLAG_CAN_ADD, 0);
 
   return obj;
 }
 
-static void session_state_changed_cb (GabbleMediaSession *session, GParamSpec *arg1, GabbleMediaChannel *channel);
-static void session_stream_added_cb (GabbleMediaSession *session, GabbleMediaStream  *stream, GabbleMediaChannel *chan);
-static void session_terminated_cb (GabbleMediaSession *session, guint terminator, guint reason, gpointer user_data);
+static void session_state_changed_cb (GabbleMediaSession *session,
+    GParamSpec *arg1, GabbleMediaChannel *channel);
+static void session_stream_added_cb (GabbleMediaSession *session,
+    GabbleMediaStream  *stream, GabbleMediaChannel *chan);
+static void session_terminated_cb (GabbleMediaSession *session,
+    guint terminator, guint reason, gpointer user_data);
 
 /**
  * create_session
@@ -203,11 +210,12 @@ static void session_terminated_cb (GabbleMediaSession *session, guint terminator
  * the "initiator" property of the newly created
  * GabbleMediaSession is set to our own handle.
  */
-static GabbleMediaSession*
+static GabbleMediaSession *
 create_session (GabbleMediaChannel *channel,
                 TpHandle peer,
                 const gchar *peer_resource,
-                const gchar *sid)
+                const gchar *sid,
+                GError **error)
 {
   GabbleMediaChannelPrivate *priv;
   GabbleMediaSession *session;
@@ -224,7 +232,37 @@ create_session (GabbleMediaChannel *channel,
 
   if (sid == NULL)
     {
+      /* We are the initiator */
+      GabblePresence *presence;
+#ifdef ENABLE_DEBUG
+      TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
+      TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (
+          conn, TP_HANDLE_TYPE_CONTACT);
+#endif
+
       initiator = INITIATOR_LOCAL;
+
+      presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
+
+      if (presence == NULL)
+        {
+          DEBUG ("failed to add contact %d (%s) to media channel: "
+              "no presence available", peer,
+              tp_handle_inspect (contact_handles, peer));
+          goto NO_CAPS;
+        }
+
+      if ((_gabble_media_channel_caps_to_typeflags (presence->caps) &
+            (TP_CHANNEL_MEDIA_CAPABILITY_AUDIO |
+             TP_CHANNEL_MEDIA_CAPABILITY_VIDEO)) == 0)
+        {
+          DEBUG ("failed to add contact %d (%s) to media channel: "
+              "caps %x aren't sufficient", peer,
+              tp_handle_inspect (contact_handles, peer),
+              presence->caps);
+          goto NO_CAPS;
+        }
+
       sid = _gabble_media_factory_allocate_sid (priv->factory, channel);
     }
   else
@@ -260,6 +298,11 @@ create_session (GabbleMediaChannel *channel,
   g_free (object_path);
 
   return session;
+
+NO_CAPS:
+  g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+      "handle %u has no media capabilities", peer);
+  return NULL;
 }
 
 gboolean
@@ -276,25 +319,30 @@ _gabble_media_channel_dispatch_session_action (GabbleMediaChannel *chan,
   GabbleMediaSession *session = priv->session;
   gboolean session_is_new = FALSE;
 
+  /* If this assertion fails, create_session() would think we're the
+   * initiator. However, GabbleMediaFactory checks this, so it can't fail */
+  g_return_val_if_fail (sid != NULL, FALSE);
+
   if (session == NULL)
     {
       TpGroupMixin *mixin = TP_GROUP_MIXIN (chan);
       TpIntSet *set;
 
-      session = create_session (chan, peer, peer_resource, sid);
+      session = create_session (chan, peer, peer_resource, sid, NULL);
+      g_assert (session != NULL);
       session_is_new = TRUE;
 
       /* make us local pending */
       set = tp_intset_new ();
       tp_intset_add (set, mixin->self_handle);
 
-      tp_group_mixin_change_members ((TpSvcChannelInterfaceGroup *)chan,
-          "", NULL, NULL, set, NULL, 0, 0);
+      tp_group_mixin_change_members ((GObject *)chan,
+          "", NULL, NULL, set, NULL, peer, 0);
 
       tp_intset_destroy (set);
 
       /* and update flags accordingly */
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)chan,
+      tp_group_mixin_change_flags ((GObject *)chan,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD | TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
           0);
     }
@@ -388,12 +436,10 @@ gabble_media_channel_set_property (GObject     *object,
       g_free (priv->object_path);
       priv->object_path = g_value_dup_string (value);
       break;
-    case PROP_HANDLE:
-      /* this property is writable in the interface, but not actually
-       * meaningfully changable on this channel, so we do nothing */
-      break;
     case PROP_HANDLE_TYPE:
-      /* this property is writable in the interface, but not actually
+    case PROP_HANDLE:
+    case PROP_CHANNEL_TYPE:
+      /* these properties are writable in the interface, but not actually
        * meaningfully changable on this channel, so we do nothing */
       break;
     case PROP_CONNECTION:
@@ -427,7 +473,8 @@ gabble_media_channel_set_property (GObject     *object,
 
 static void gabble_media_channel_dispose (GObject *object);
 static void gabble_media_channel_finalize (GObject *object);
-static gboolean gabble_media_channel_remove_member (TpSvcChannelInterfaceGroup *obj, TpHandle handle, const gchar *message, GError **error);
+static gboolean gabble_media_channel_remove_member (GObject *obj,
+    TpHandle handle, const gchar *message, GError **error);
 
 static void
 gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_class)
@@ -435,7 +482,8 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_channel_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_media_channel_class, sizeof (GabbleMediaChannelPrivate));
+  g_type_class_add_private (gabble_media_channel_class,
+      sizeof (GabbleMediaChannelPrivate));
 
   object_class->constructor = gabble_media_channel_constructor;
 
@@ -445,14 +493,16 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
   object_class->dispose = gabble_media_channel_dispose;
   object_class->finalize = gabble_media_channel_finalize;
 
-  tp_group_mixin_class_init ((TpSvcChannelInterfaceGroupClass *)object_class,
-                                 G_STRUCT_OFFSET (GabbleMediaChannelClass, group_class),
-                                 _gabble_media_channel_add_member,
-                                 gabble_media_channel_remove_member);
+  tp_group_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (GabbleMediaChannelClass, group_class),
+      _gabble_media_channel_add_member, gabble_media_channel_remove_member);
 
-  g_object_class_override_property (object_class, PROP_OBJECT_PATH, "object-path");
-  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE, "channel-type");
-  g_object_class_override_property (object_class, PROP_HANDLE_TYPE, "handle-type");
+  g_object_class_override_property (object_class, PROP_OBJECT_PATH,
+      "object-path");
+  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE,
+      "channel-type");
+  g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
+      "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
 
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
@@ -492,7 +542,8 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
                                     G_PARAM_READWRITE |
                                     G_PARAM_STATIC_NAME |
                                     G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_NAT_TRAVERSAL, param_spec);
+  g_object_class_install_property (object_class, PROP_NAT_TRAVERSAL,
+      param_spec);
 
   param_spec = g_param_spec_string ("stun-server",
                                     "STUN server",
@@ -520,7 +571,8 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
                                     G_PARAM_READWRITE |
                                     G_PARAM_STATIC_NAME |
                                     G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_GTALK_P2P_RELAY_TOKEN, param_spec);
+  g_object_class_install_property (object_class, PROP_GTALK_P2P_RELAY_TOKEN,
+      param_spec);
 
   tp_properties_mixin_class_init (object_class,
       G_STRUCT_OFFSET (GabbleMediaChannelClass, properties_class),
@@ -561,7 +613,8 @@ gabble_media_channel_finalize (GObject *object)
 
   g_free (priv->object_path);
 
-  tp_group_mixin_finalize ((TpSvcChannelInterfaceGroup *)object);
+  tp_group_mixin_finalize (object);
+  tp_properties_mixin_finalize (object);
 
   G_OBJECT_CLASS (gabble_media_channel_parent_class)->finalize (object);
 }
@@ -603,7 +656,8 @@ gabble_media_channel_close (GabbleMediaChannel *self)
 
   if (priv->session)
     {
-      _gabble_media_session_terminate (priv->session, INITIATOR_LOCAL, TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
+      _gabble_media_session_terminate (priv->session, INITIATOR_LOCAL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
     }
 
   tp_svc_channel_emit_closed (self);
@@ -650,7 +704,12 @@ gabble_media_channel_get_interfaces (TpSvcChannel *iface,
                                      DBusGMethodInvocation *context)
 {
   const gchar *interfaces[] = {
+      /* FIXME: our implementation of CallState is a stub, so it doesn't
+      appear in GetInterfaces' output to avoid confusing clients
+      TP_IFACE_CHANNEL_INTERFACE_CALL_STATE,
+      */
       TP_IFACE_CHANNEL_INTERFACE_GROUP,
+      TP_IFACE_CHANNEL_INTERFACE_HOLD,
       TP_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING,
       TP_IFACE_PROPERTIES_INTERFACE,
       NULL
@@ -684,9 +743,9 @@ gabble_media_channel_get_session_handlers (TpSvcChannelInterfaceMediaSignalling
       TpHandle member;
       gchar *path;
 
-      g_value_init (&handler, TP_SESSION_HANDLER_SET_TYPE);
+      g_value_init (&handler, GABBLE_TP_SESSION_HANDLER_SET_TYPE);
       g_value_take_boxed (&handler,
-          dbus_g_type_specialized_construct (TP_SESSION_HANDLER_SET_TYPE));
+          dbus_g_type_specialized_construct (GABBLE_TP_SESSION_HANDLER_SET_TYPE));
 
       g_object_get (priv->session,
                     "peer", &member,
@@ -743,9 +802,9 @@ make_stream_list (GabbleMediaChannel *self,
 
       g_object_get (priv->session, "peer", &peer, NULL);
 
-      g_value_init (&entry, TP_CHANNEL_STREAM_TYPE);
+      g_value_init (&entry, GABBLE_TP_CHANNEL_STREAM_TYPE);
       g_value_take_boxed (&entry,
-          dbus_g_type_specialized_construct (TP_CHANNEL_STREAM_TYPE));
+          dbus_g_type_specialized_construct (GABBLE_TP_CHANNEL_STREAM_TYPE));
 
       dbus_g_type_struct_set (&entry,
           0, id,
@@ -858,7 +917,8 @@ gabble_media_channel_remove_streams (TpSvcChannelTypeStreamedMedia *iface,
           goto OUT;
         }
 
-      /* make sure we don't allow the client to repeatedly remove the same stream */
+      /* make sure we don't allow the client to repeatedly remove the same
+      stream */
       for (j = 0; j < stream_objs->len; j++)
         {
           GabbleMediaStream *tmp = g_ptr_array_index (stream_objs, j);
@@ -969,40 +1029,52 @@ gabble_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
   GPtrArray *streams;
   GError *error = NULL;
   GPtrArray *ret;
+  TpHandleRepoIface *contact_handles;
 
   g_assert (GABBLE_IS_MEDIA_CHANNEL (self));
 
+  /* FIXME: disallow this if we've put the other guy on hold? */
+
   priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (self);
   conn = (TpBaseConnection *)priv->conn;
+  contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
 
-  if (!tp_handle_is_valid (conn->handles[TP_HANDLE_TYPE_CONTACT],
-        contact_handle, &error))
+  if (!tp_handle_is_valid (contact_handles, contact_handle, &error))
+    goto error;
+
+  if (priv->session == NULL)
     {
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      return;
+      if (create_session (self, contact_handle, NULL, NULL, &error)
+          == NULL)
+        {
+          dbus_g_method_return_error (context, error);
+          g_error_free (error);
+          return;
+        }
+    }
+  else
+    {
+      TpHandle peer;
+
+      g_object_get (priv->session,
+          "peer", &peer,
+          NULL);
+
+      if (peer != contact_handle)
+        {
+          g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+              "cannot add streams for %u: this channel's peer is %u",
+              contact_handle, peer);
+          goto error;
+        }
     }
 
-  if (!tp_handle_set_is_member (self->group.members, contact_handle) &&
-      !tp_handle_set_is_member (self->group.remote_pending, contact_handle))
-    {
-      g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
-          "given handle %u is not a member of the channel", contact_handle);
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      return;
-    }
-
-  /* if the person is a channel member, we should have a session */
   g_assert (priv->session != NULL);
 
   if (!_gabble_media_session_request_streams (priv->session, types, &streams,
         &error))
-    {
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      return;
-    }
+    goto error;
 
   ret = make_stream_list (self, streams);
 
@@ -1010,11 +1082,16 @@ gabble_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
 
   tp_svc_channel_type_streamed_media_return_from_request_streams (context, ret);
   g_ptr_array_free (ret, TRUE);
+  return;
+
+error:
+  dbus_g_method_return_error (context, error);
+  g_error_free (error);
 }
 
 
 gboolean
-_gabble_media_channel_add_member (TpSvcChannelInterfaceGroup *obj,
+_gabble_media_channel_add_member (GObject *obj,
                                   TpHandle handle,
                                   const gchar *message,
                                   GError **error)
@@ -1022,43 +1099,36 @@ _gabble_media_channel_add_member (TpSvcChannelInterfaceGroup *obj,
   GabbleMediaChannel *chan = GABBLE_MEDIA_CHANNEL (obj);
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
   TpGroupMixin *mixin = TP_GROUP_MIXIN (obj);
-#ifdef ENABLE_DEBUG
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
-#endif
 
   /* did we create this channel? */
   if (priv->creator == mixin->self_handle)
     {
-      GabblePresence *presence;
       TpIntSet *set;
 
-      /* yes: check the peer's capabilities */
-
-      presence = gabble_presence_cache_get (priv->conn->presence_cache, handle);
-
-      if (presence == NULL)
-        {
-          DEBUG ("failed to add contact %d (%s) to media channel: "
-              "no presence available", handle, tp_handle_inspect (
-                conn->handles[TP_HANDLE_TYPE_CONTACT], handle));
-          goto NO_CAPS;
-        }
-
-      if (!(presence->caps & PRESENCE_CAP_GOOGLE_VOICE ||
-            presence->caps & PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO ||
-            presence->caps & PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO))
-        {
-          DEBUG ("failed to add contact %d (%s) to media channel: "
-              "caps %x aren't sufficient", handle, tp_handle_inspect (
-                conn->handles[TP_HANDLE_TYPE_CONTACT], handle),
-              presence->caps);
-          goto NO_CAPS;
-        }
-
       /* yes: invite the peer */
 
-      /* create a new session */
-      create_session (chan, handle, NULL, NULL);
+      if (priv->session == NULL)
+        {
+          /* create a new session */
+          if (create_session (chan, handle, NULL, NULL, error) == NULL)
+            return FALSE;
+        }
+      else
+        {
+          TpHandle peer;
+
+          g_object_get (priv->session,
+              "peer", &peer,
+              NULL);
+
+          if (peer != handle)
+            {
+              g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+                  "handle %u cannot be added: this channel's peer is %u",
+                  handle, peer);
+              return FALSE;
+            }
+        }
 
       /* make the peer remote pending */
       set = tp_intset_new ();
@@ -1111,15 +1181,13 @@ _gabble_media_channel_add_member (TpSvcChannelInterfaceGroup *obj,
   g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
       "handle %u cannot be added in the current state", handle);
   return FALSE;
-
-NO_CAPS:
-  g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-      "handle %u has no media capabilities", handle);
-  return FALSE;
 }
 
 static gboolean
-gabble_media_channel_remove_member (TpSvcChannelInterfaceGroup *obj, TpHandle handle, const gchar *message, GError **error)
+gabble_media_channel_remove_member (GObject *obj,
+                                    TpHandle handle,
+                                    const gchar *message,
+                                    GError **error)
 {
   GabbleMediaChannel *chan = GABBLE_MEDIA_CHANNEL (obj);
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
@@ -1144,7 +1212,8 @@ gabble_media_channel_remove_member (TpSvcChannelInterfaceGroup *obj, TpHandle ha
       return FALSE;
     }
 
-  _gabble_media_session_terminate (priv->session, INITIATOR_LOCAL, TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
+  _gabble_media_session_terminate (priv->session, INITIATOR_LOCAL,
+      TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
 
   /* remove the member */
   set = tp_intset_new ();
@@ -1186,11 +1255,11 @@ session_terminated_cb (GabbleMediaSession *session,
   tp_intset_add (set, mixin->self_handle);
   tp_intset_add (set, peer);
 
-  tp_group_mixin_change_members ((TpSvcChannelInterfaceGroup *)channel,
+  tp_group_mixin_change_members ((GObject *)channel,
       "", NULL, set, NULL, NULL, terminator, reason);
 
   /* update flags accordingly -- allow adding, deny removal */
-  tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)channel,
+  tp_group_mixin_change_flags ((GObject *)channel,
       TP_CHANNEL_GROUP_FLAG_CAN_ADD,
       TP_CHANNEL_GROUP_FLAG_CAN_REMOVE);
 
@@ -1236,28 +1305,222 @@ session_state_changed_cb (GabbleMediaSession *session,
                 "peer", &peer,
                 NULL);
 
-  if (state != JS_STATE_ACTIVE)
-    return;
-
-  if (priv->creator != mixin->self_handle)
-    return;
-
   set = tp_intset_new ();
 
-  /* add the peer to the member list */
   tp_intset_add (set, peer);
 
-  tp_group_mixin_change_members ((TpSvcChannelInterfaceGroup *)channel,
-      "", set, NULL, NULL, NULL, 0, 0);
+  if (state >= JS_STATE_PENDING_INITIATE_SENT &&
+      state < JS_STATE_ACTIVE &&
+      !tp_handle_set_is_member (mixin->members, peer))
+    {
+      /* The first time we send anything to the other user, they materialise
+       * in remote-pending if necessary */
 
-  /* update flags accordingly -- allow removal, deny adding and rescinding */
-  tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)channel,
-      TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
-      TP_CHANNEL_GROUP_FLAG_CAN_ADD | TP_CHANNEL_GROUP_FLAG_CAN_RESCIND);
+      tp_group_mixin_change_members ((GObject *) channel,
+          "", NULL, NULL, NULL, set, 0, 0);
+
+      tp_group_mixin_change_flags ((GObject *) channel,
+          TP_CHANNEL_GROUP_FLAG_CAN_REMOVE | TP_CHANNEL_GROUP_FLAG_CAN_RESCIND,
+          TP_CHANNEL_GROUP_FLAG_CAN_ADD);
+    }
+
+  if (state == JS_STATE_ACTIVE &&
+      priv->creator == mixin->self_handle)
+    {
+      /* add the peer to the member list */
+      tp_group_mixin_change_members ((GObject *)channel,
+          "", set, NULL, NULL, NULL, 0, 0);
+
+      /* update flags accordingly -- allow removal, deny adding and
+       * rescinding */
+      tp_group_mixin_change_flags ((GObject *) channel,
+          TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
+          TP_CHANNEL_GROUP_FLAG_CAN_ADD | TP_CHANNEL_GROUP_FLAG_CAN_RESCIND);
+    }
 
   tp_intset_destroy (set);
 }
 
+
+static void
+inform_peer_of_unhold (GabbleMediaChannel *self)
+{
+  /* FIXME: when we upgrade to current Jingle, signal to the peer that
+   * we've taken them off hold, via a session-info message;
+   * ignore success or failure, since there's nothing we could really
+   * do differently, and the message is only advisory.
+   *
+   * For now, we don't signal the unhold in the XMPP stream */
+  DEBUG ("TODO: tell peer we've taken them off hold");
+}
+
+
+static void
+inform_peer_of_hold (GabbleMediaChannel *self)
+{
+  /* FIXME: when we upgrade to current Jingle, signal to the peer that
+   * we're putting them on hold, via a session-info message;
+   * ignore success or failure, since there's nothing we could really
+   * do differently, and the message is only advisory.
+   *
+   * For now, we don't signal the hold in the XMPP stream */
+  DEBUG ("TODO: tell peer we're putting them on hold");
+}
+
+
+static void
+stream_hold_state_changed (GabbleMediaStream *stream G_GNUC_UNUSED,
+                           GParamSpec *unused G_GNUC_UNUSED,
+                           gpointer data)
+{
+  GabbleMediaChannel *self = data;
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (self);
+  gboolean all_held = TRUE, any_held = FALSE;
+  guint i;
+
+  for (i = 0; i < priv->streams->len; i++)
+    {
+      gboolean its_hold;
+
+      g_object_get (g_ptr_array_index (priv->streams, i),
+          "local-hold", &its_hold,
+          NULL);
+
+      DEBUG ("Stream at index %u has local-hold=%u", i, (guint) its_hold);
+
+      all_held = all_held && its_hold;
+      any_held = any_held || its_hold;
+    }
+
+  DEBUG ("all_held=%u, any_held=%u", (guint) all_held, (guint) any_held);
+
+  if (all_held)
+    {
+      /* Move to state HELD */
+
+      if (priv->hold_state == TP_LOCAL_HOLD_STATE_HELD)
+        {
+          /* nothing changed */
+          return;
+        }
+      else if (priv->hold_state == TP_LOCAL_HOLD_STATE_PENDING_UNHOLD)
+        {
+          /* This can happen if the user asks us to hold, then changes their
+           * mind. We make no particular guarantees about stream states when
+           * in PENDING_UNHOLD state, so keep claiming to be in that state */
+          return;
+        }
+      else if (priv->hold_state == TP_LOCAL_HOLD_STATE_PENDING_HOLD)
+        {
+          /* We wanted to hold, and indeed we have. Yay! Keep whatever
+           * reason code we used for going to PENDING_HOLD */
+          priv->hold_state = TP_LOCAL_HOLD_STATE_HELD;
+        }
+      else
+        {
+          /* We were previously UNHELD. So why have we gone on hold now? */
+          DEBUG ("Unexpectedly entered HELD state!");
+          priv->hold_state = TP_LOCAL_HOLD_STATE_HELD;
+          priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_NONE;
+        }
+    }
+  else if (any_held)
+    {
+      if (priv->hold_state == TP_LOCAL_HOLD_STATE_UNHELD)
+        {
+          /* The streaming client has spontaneously changed its stream
+           * state. Why? We just don't know */
+          DEBUG ("Unexpectedly entered PENDING_UNHOLD state!");
+          priv->hold_state = TP_LOCAL_HOLD_STATE_PENDING_UNHOLD;
+          priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_NONE;
+        }
+      else if (priv->hold_state == TP_LOCAL_HOLD_STATE_HELD)
+        {
+          /* Likewise */
+          DEBUG ("Unexpectedly entered PENDING_HOLD state!");
+          priv->hold_state = TP_LOCAL_HOLD_STATE_PENDING_HOLD;
+          priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_NONE;
+        }
+      else
+        {
+          /* nothing particularly interesting - we're trying to change hold
+           * state already, so nothing to signal */
+          return;
+        }
+
+      /* Tell the peer what's happened */
+      inform_peer_of_unhold (self);
+    }
+  else
+    {
+      /* Move to state UNHELD */
+
+      if (priv->hold_state == TP_LOCAL_HOLD_STATE_UNHELD)
+        {
+          /* nothing changed */
+          return;
+        }
+      else if (priv->hold_state == TP_LOCAL_HOLD_STATE_PENDING_HOLD)
+        {
+          /* This can happen if the user asks us to unhold, then changes their
+           * mind. We make no particular guarantees about stream states when
+           * in PENDING_HOLD state, so keep claiming to be in that state */
+          return;
+        }
+      else if (priv->hold_state == TP_LOCAL_HOLD_STATE_PENDING_UNHOLD)
+        {
+          /* We wanted to hold, and indeed we have. Yay! Keep whatever
+           * reason code we used for going to PENDING_UNHOLD */
+          priv->hold_state = TP_LOCAL_HOLD_STATE_UNHELD;
+        }
+      else
+        {
+          /* We were previously HELD. So why have we gone off hold now? */
+          DEBUG ("Unexpectedly entered UNHELD state!");
+          priv->hold_state = TP_LOCAL_HOLD_STATE_UNHELD;
+          priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_NONE;
+        }
+    }
+
+  tp_svc_channel_interface_hold_emit_hold_state_changed (self,
+      priv->hold_state, priv->hold_state_reason);
+}
+
+
+static void
+stream_unhold_failed (GabbleMediaStream *stream,
+                      gpointer data)
+{
+  GabbleMediaChannel *self = data;
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (self);
+  guint i;
+
+  DEBUG ("%p: %p", self, stream);
+
+  /* Unholding failed - let's roll back to Hold state */
+  priv->hold_state = TP_LOCAL_HOLD_STATE_PENDING_HOLD;
+  priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_RESOURCE_NOT_AVAILABLE;
+  tp_svc_channel_interface_hold_emit_hold_state_changed (self,
+      priv->hold_state, priv->hold_state_reason);
+
+  /* The stream's state may have changed from unheld to held, so re-poll.
+   * It's possible that all streams are now held, in which case we can stop. */
+  stream_hold_state_changed (stream, NULL, self);
+
+  if (priv->hold_state == TP_LOCAL_HOLD_STATE_HELD)
+    return;
+
+  /* There should be no need to notify the peer, who already thinks they're
+   * on hold, so just tell the streaming client what to do. */
+
+  for (i = 0; i < priv->streams->len; i++)
+    {
+      gabble_media_stream_hold (g_ptr_array_index (priv->streams, i),
+          TRUE);
+    }
+}
+
+
 static void
 stream_close_cb (GabbleMediaStream *stream,
                  GabbleMediaChannel *chan)
@@ -1265,13 +1528,21 @@ stream_close_cb (GabbleMediaStream *stream,
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
   guint id;
 
-  g_object_get (stream, "id", &id, NULL);
+  g_object_get (stream,
+      "id", &id,
+      NULL);
 
   tp_svc_channel_type_streamed_media_emit_stream_removed (chan, id);
 
   if (priv->streams != NULL)
     {
       g_ptr_array_remove (priv->streams, stream);
+
+      /* A stream closing might cause the "total" hold state to change:
+       * if there's one held and one unheld, and the unheld one closes,
+       * then our state changes from indeterminate to held. */
+      stream_hold_state_changed (stream, NULL, chan);
+
       g_object_unref (stream);
     }
 }
@@ -1302,7 +1573,10 @@ stream_state_changed_cb (GabbleMediaStream *stream,
   guint id;
   TpMediaStreamState connection_state;
 
-  g_object_get (stream, "id", &id, "connection-state", &connection_state, NULL);
+  g_object_get (stream,
+      "id", &id,
+      "connection-state", &connection_state,
+      NULL);
 
   tp_svc_channel_type_streamed_media_emit_stream_state_changed (chan,
       id, connection_state);
@@ -1347,10 +1621,14 @@ session_stream_added_cb (GabbleMediaSession *session,
                     (GCallback) stream_close_cb, chan);
   g_signal_connect (stream, "error",
                     (GCallback) stream_error_cb, chan);
+  g_signal_connect (stream, "unhold-failed",
+                    (GCallback) stream_unhold_failed, chan);
   g_signal_connect (stream, "notify::connection-state",
                     (GCallback) stream_state_changed_cb, chan);
   g_signal_connect (stream, "notify::combined-direction",
                     (GCallback) stream_direction_changed_cb, chan);
+  g_signal_connect (stream, "notify::local-hold",
+                    (GCallback) stream_hold_state_changed, chan);
 
   /* emit StreamAdded */
   g_object_get (session, "peer", &handle, NULL);
@@ -1358,6 +1636,9 @@ session_stream_added_cb (GabbleMediaSession *session,
 
   tp_svc_channel_type_streamed_media_emit_stream_added (
       chan, id, handle, type);
+
+  /* A stream being added might cause the "total" hold state to change */
+  stream_hold_state_changed (stream, NULL, chan);
 }
 
 guint
@@ -1368,10 +1649,17 @@ _gabble_media_channel_get_stream_id (GabbleMediaChannel *chan)
   return priv->next_stream_id++;
 }
 
-#define AUDIO_CAPS \
-  ( PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO )
+#define GTALK_CAPS \
+  ( PRESENCE_CAP_GOOGLE_VOICE )
 
-#define VIDEO_CAPS \
+#define JINGLE_CAPS \
+  ( PRESENCE_CAP_JINGLE \
+  | PRESENCE_CAP_GOOGLE_TRANSPORT_P2P )
+
+#define JINGLE_AUDIO_CAPS \
+  ( PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO )
+
+#define JINGLE_VIDEO_CAPS \
   ( PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO )
 
 GabblePresenceCapabilities
@@ -1379,11 +1667,18 @@ _gabble_media_channel_typeflags_to_caps (TpChannelMediaCapabilities flags)
 {
   GabblePresenceCapabilities caps = 0;
 
-  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_AUDIO)
-    caps |= AUDIO_CAPS;
+  /* currently we can only signal any (GTalk or Jingle calls) using
+   * the GTalk-P2P transport */
+  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P)
+    {
+      caps |= JINGLE_CAPS;
 
-  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_VIDEO)
-    caps |= VIDEO_CAPS;
+      if (flags & TP_CHANNEL_MEDIA_CAPABILITY_AUDIO)
+        caps |= GTALK_CAPS | JINGLE_AUDIO_CAPS;
+
+      if (flags & TP_CHANNEL_MEDIA_CAPABILITY_VIDEO)
+        caps |= JINGLE_VIDEO_CAPS;
+    }
 
   return caps;
 }
@@ -1393,18 +1688,108 @@ _gabble_media_channel_caps_to_typeflags (GabblePresenceCapabilities caps)
 {
   TpChannelMediaCapabilities typeflags = 0;
 
-  if (caps & AUDIO_CAPS)
+  /* this is intentionally asymmetric to the previous function - we don't
+   * require the other end to advertise the GTalk-P2P transport capability
+   * separately because old GTalk clients didn't do that - having Google voice
+   * implied Google session and GTalk-P2P */
+  if ((caps & GTALK_CAPS) == GTALK_CAPS)
     typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_AUDIO;
 
-  if (caps & VIDEO_CAPS)
-    typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_VIDEO;
+  if ((caps & JINGLE_CAPS) == JINGLE_CAPS)
+    {
+      if ((caps & JINGLE_AUDIO_CAPS) == JINGLE_AUDIO_CAPS)
+        typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_AUDIO;
+
+      if ((caps & JINGLE_VIDEO_CAPS) == JINGLE_VIDEO_CAPS)
+        typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_VIDEO;
+    }
 
   return typeflags;
 }
 
+static void
+gabble_media_channel_get_call_states (TpSvcChannelInterfaceCallState *iface,
+                                      DBusGMethodInvocation *context)
+{
+  GHashTable *states;
+
+  /* stub implementation: nobody has any call-state flags */
+  states = g_hash_table_new (g_direct_hash, g_direct_equal);
+  tp_svc_channel_interface_call_state_return_from_get_call_states (context,
+      states);
+  g_hash_table_destroy (states);
+}
 
 static void
-channel_iface_init(gpointer g_iface, gpointer iface_data)
+gabble_media_channel_get_hold_state (TpSvcChannelInterfaceHold *iface,
+                                     DBusGMethodInvocation *context)
+{
+  GabbleMediaChannel *self = (GabbleMediaChannel *) iface;
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (self);
+
+  tp_svc_channel_interface_hold_return_from_get_hold_state (context,
+      priv->hold_state, priv->hold_state_reason);
+}
+
+
+static void
+gabble_media_channel_request_hold (TpSvcChannelInterfaceHold *iface,
+                                   gboolean hold,
+                                   DBusGMethodInvocation *context)
+{
+  GabbleMediaChannel *self = GABBLE_MEDIA_CHANNEL (iface);
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (self);
+  guint i;
+  TpLocalHoldState old_state = priv->hold_state;
+
+  DEBUG ("%p: RequestHold(%u)", self, !!hold);
+
+  if (hold)
+    {
+      if (priv->hold_state == TP_LOCAL_HOLD_STATE_HELD)
+        {
+          DEBUG ("No-op");
+          tp_svc_channel_interface_hold_return_from_request_hold (context);
+          return;
+        }
+
+      inform_peer_of_hold (self);
+
+      priv->hold_state = TP_LOCAL_HOLD_STATE_PENDING_HOLD;
+    }
+  else
+    {
+      if (priv->hold_state == TP_LOCAL_HOLD_STATE_UNHELD)
+        {
+          DEBUG ("No-op");
+          tp_svc_channel_interface_hold_return_from_request_hold (context);
+          return;
+        }
+
+      priv->hold_state = TP_LOCAL_HOLD_STATE_PENDING_UNHOLD;
+    }
+
+  if (old_state != priv->hold_state ||
+      priv->hold_state_reason != TP_LOCAL_HOLD_STATE_REASON_REQUESTED)
+    {
+      tp_svc_channel_interface_hold_emit_hold_state_changed (self,
+          priv->hold_state, TP_LOCAL_HOLD_STATE_REASON_REQUESTED);
+      priv->hold_state_reason = TP_LOCAL_HOLD_STATE_REASON_REQUESTED;
+    }
+
+  /* Tell streaming client to release or reacquire resources */
+
+  for (i = 0; i < priv->streams->len; i++)
+    {
+      gabble_media_stream_hold (g_ptr_array_index (priv->streams, i), hold);
+    }
+
+  tp_svc_channel_interface_hold_return_from_request_hold (context);
+}
+
+
+static void
+channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
@@ -1418,9 +1803,10 @@ channel_iface_init(gpointer g_iface, gpointer iface_data)
 }
 
 static void
-streamed_media_iface_init(gpointer g_iface, gpointer iface_data)
+streamed_media_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelTypeStreamedMediaClass *klass = (TpSvcChannelTypeStreamedMediaClass *)g_iface;
+  TpSvcChannelTypeStreamedMediaClass *klass =
+    (TpSvcChannelTypeStreamedMediaClass *)g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_type_streamed_media_implement_##x (\
     klass, gabble_media_channel_##x)
@@ -1432,12 +1818,38 @@ streamed_media_iface_init(gpointer g_iface, gpointer iface_data)
 }
 
 static void
-media_signalling_iface_init(gpointer g_iface, gpointer iface_data)
+media_signalling_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelInterfaceMediaSignallingClass *klass = (TpSvcChannelInterfaceMediaSignallingClass *)g_iface;
+  TpSvcChannelInterfaceMediaSignallingClass *klass =
+    (TpSvcChannelInterfaceMediaSignallingClass *)g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_interface_media_signalling_implement_##x (\
     klass, gabble_media_channel_##x)
   IMPLEMENT(get_session_handlers);
 #undef IMPLEMENT
 }
+
+static void
+call_state_iface_init (gpointer g_iface,
+                       gpointer iface_data G_GNUC_UNUSED)
+{
+  TpSvcChannelInterfaceCallStateClass *klass = g_iface;
+
+#define IMPLEMENT(x) tp_svc_channel_interface_call_state_implement_##x (\
+    klass, gabble_media_channel_##x)
+  IMPLEMENT(get_call_states);
+#undef IMPLEMENT
+}
+
+static void
+hold_iface_init (gpointer g_iface,
+                 gpointer iface_data G_GNUC_UNUSED)
+{
+  TpSvcChannelInterfaceHoldClass *klass = g_iface;
+
+#define IMPLEMENT(x) tp_svc_channel_interface_hold_implement_##x (\
+    klass, gabble_media_channel_##x)
+  IMPLEMENT(get_hold_state);
+  IMPLEMENT(request_hold);
+#undef IMPLEMENT
+}
