@@ -296,6 +296,7 @@ gabble_media_session_constructor (GType type, guint n_props,
   GObject *obj;
   GabbleMediaSessionPrivate *priv;
   DBusGConnection *bus;
+  TpHandleRepoIface *contact_handles;
 
   obj = G_OBJECT_CLASS (gabble_media_session_parent_class)->
            constructor (type, n_props, props);
@@ -304,6 +305,11 @@ gabble_media_session_constructor (GType type, guint n_props,
   bus = tp_get_bus ();
   dbus_g_connection_register_g_object (bus, priv->object_path, obj);
 
+  contact_handles = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+
+  tp_handle_ref (contact_handles, priv->peer);
+
   return obj;
 }
 
@@ -530,6 +536,7 @@ gabble_media_session_dispose (GObject *object)
   GabbleMediaSession *self = GABBLE_MEDIA_SESSION (object);
   GabbleMediaSessionPrivate *priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (self);
   guint i;
+  TpHandleRepoIface *contact_handles;
 
   DEBUG ("called");
 
@@ -557,6 +564,11 @@ gabble_media_session_dispose (GObject *object)
   g_ptr_array_free (priv->remove_requests, TRUE);
   priv->remove_requests = NULL;
 
+  contact_handles = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+
+  tp_handle_unref (contact_handles, priv->peer);
+
   if (G_OBJECT_CLASS (gabble_media_session_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_media_session_parent_class)->dispose (object);
 }
@@ -641,17 +653,17 @@ gabble_media_session_ready (TpSvcMediaSessionHandler *iface,
                             DBusGMethodInvocation *context)
 {
   GabbleMediaSession *self = GABBLE_MEDIA_SESSION (iface);
-  GabbleMediaSessionPrivate *priv;
-  guint i;
+  GabbleMediaSessionPrivate *priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (self);
 
-  g_assert (GABBLE_IS_MEDIA_SESSION (self));
+  if (!priv->ready)
+    {
+      guint i;
 
-  priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (self);
+      priv->ready = TRUE;
 
-  priv->ready = TRUE;
-
-  for (i = 0; i < priv->streams->len; i++)
-    _emit_new_stream (self, g_ptr_array_index (priv->streams, i));
+      for (i = 0; i < priv->streams->len; i++)
+        _emit_new_stream (self, g_ptr_array_index (priv->streams, i));
+    }
 
   tp_svc_media_session_handler_return_from_ready (context);
 }
@@ -1415,7 +1427,7 @@ timeout_session (gpointer data)
 
 static void do_content_add (GabbleMediaSession *, GabbleMediaStream *);
 
-void
+static void
 _add_ready_new_streams (GabbleMediaSession *session)
 {
   GabbleMediaSessionPrivate *priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (session);
@@ -2271,13 +2283,12 @@ _gabble_media_session_remove_streams (GabbleMediaSession *session,
         case STREAM_SIG_STATE_SENT:
         case STREAM_SIG_STATE_ACKNOWLEDGED:
           {
-            LmMessageNode *content_node;
-
+            g_assert (priv->state > JS_STATE_PENDING_CREATED);
             g_assert (msg != NULL);
             g_assert (removing != NULL);
+            g_assert (session_node != NULL);
 
-            content_node = _gabble_media_stream_add_content_node (stream,
-                session_node);
+            _gabble_media_stream_add_content_node (stream, session_node);
 
             g_object_set (stream,
                 "playing", FALSE,
@@ -2321,6 +2332,10 @@ _gabble_media_session_remove_streams (GabbleMediaSession *session,
       GMS_DEBUG_INFO (session, "not sending jingle session action "
           "\"content-remove\" to peer, no initiates or adds sent for "
           "these streams");
+      if (priv->state < JS_STATE_PENDING_INITIATE_SENT)
+        {
+           try_session_initiate (session);
+        }
     }
 }
 
@@ -2423,7 +2438,7 @@ _gabble_media_session_terminate (GabbleMediaSession *session,
   g_signal_emit (session, signals[TERMINATED], 0, actor, why);
 }
 
-#if _GMS_DEBUG_LEVEL
+#if defined (ENABLE_DEBUG) && _GMS_DEBUG_LEVEL
 void
 _gabble_media_session_debug (GabbleMediaSession *session,
                              DebugMessageType type,
@@ -2529,6 +2544,9 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
                                        GPtrArray **ret,
                                        GError **error)
 {
+  /* if you change the caps in this function, you almost certainly also need
+   * to change _gabble_media_channel_typeflags_to_caps and
+   * _gabble_media_channel_caps_to_typeflags in gabble-media-channel.c */
   static GabblePresenceCapabilities google_audio_caps =
     PRESENCE_CAP_GOOGLE_VOICE;
   static GabblePresenceCapabilities jingle_audio_caps =
