@@ -220,6 +220,9 @@ struct _GabbleMucChannelPrivate
   gboolean dispose_has_run;
 
   gboolean invite_self;
+
+  /* Aggregate all presences when joining the chatroom */
+  TpHandleSet *initial_members_aggregator;
 };
 
 #define GABBLE_MUC_CHANNEL_GET_PRIVATE(o) \
@@ -229,7 +232,10 @@ struct _GabbleMucChannelPrivate
 static void
 gabble_muc_channel_init (GabbleMucChannel *obj)
 {
-  /* do nothing? */
+  GabbleMucChannelPrivate *priv;
+
+  priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (obj);
+  priv->initial_members_aggregator = NULL;
 }
 
 static void contact_handle_to_room_identity (GabbleMucChannel *, TpHandle,
@@ -268,6 +274,8 @@ gabble_muc_channel_constructor (GType type, guint n_props,
 
   tp_handle_ref (contact_handles, self_handle);
 
+  priv->initial_members_aggregator = tp_handle_set_new (contact_handles);
+
   /* initialize our own role and affiliation */
   priv->self_role = ROLE_NONE;
   priv->self_affil = AFFILIATION_NONE;
@@ -326,7 +334,6 @@ properties_disco_cb (GabbleDisco *disco,
                      gpointer user_data)
 {
   GabbleMucChannel *chan = user_data;
-  GabbleMucChannelPrivate *priv;
   TpIntSet *changed_props_val, *changed_props_flags;
   LmMessageNode *lm_node;
   const gchar *str;
@@ -334,8 +341,6 @@ properties_disco_cb (GabbleDisco *disco,
 
   g_assert (GABBLE_IS_MUC_CHANNEL (chan));
 
-  priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (chan);
-
   if (error)
     {
       DEBUG ("got error %s", error->message);
@@ -931,6 +936,12 @@ gabble_muc_channel_finalize (GObject *object)
 
   DEBUG ("called");
 
+  if (priv->initial_members_aggregator != NULL)
+    {
+      tp_handle_set_destroy (priv->initial_members_aggregator);
+      priv->initial_members_aggregator = NULL;
+    }
+
   /* free any data held directly by the object here */
   tp_handle_unref (room_handles, priv->handle);
 
@@ -977,7 +988,7 @@ static void clear_poll_timer (GabbleMucChannel *chan)
 static void
 change_password_flags (GabbleMucChannel *chan,
                        TpChannelPasswordFlags add,
-                       TpChannelPasswordFlags remove)
+                       TpChannelPasswordFlags del)
 {
   GabbleMucChannelPrivate *priv;
   TpChannelPasswordFlags added, removed;
@@ -989,10 +1000,10 @@ change_password_flags (GabbleMucChannel *chan,
   added = add & ~priv->password_flags;
   priv->password_flags |= added;
 
-  removed = remove & priv->password_flags;
+  removed = del & priv->password_flags;
   priv->password_flags &= ~removed;
 
-  if (add != 0 || remove != 0)
+  if (add != 0 || del != 0)
     {
       DEBUG ("emitting password flags changed, added 0x%X, removed 0x%X",
               added, removed);
@@ -1597,6 +1608,7 @@ _gabble_muc_channel_member_presence_updated (GabbleMucChannel *chan,
   DEBUG ("called");
 
   g_assert (GABBLE_IS_MUC_CHANNEL (chan));
+  g_assert (handle != 0);
 
   priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (chan);
   conn = (TpBaseConnection *)priv->conn;
@@ -1634,8 +1646,31 @@ _gabble_muc_channel_member_presence_updated (GabbleMucChannel *chan,
     {
       if (!tp_handle_set_is_member (mixin->members, handle))
         {
-          tp_group_mixin_change_members ((GObject *)chan, "", set, NULL,
-                                             NULL, NULL, 0, 0);
+          if (priv->initial_members_aggregator == NULL)
+            {
+              /* we've already had the initial batch of presence stanzas */
+              tp_group_mixin_change_members ((GObject *) chan, "", set, NULL,
+                                              NULL, NULL, 0, 0);
+            }
+          else
+            {
+              /* aggregate this presence */
+              tp_handle_set_add (priv->initial_members_aggregator, handle);
+
+              /* Do not emit one signal per presence. Instead, get all
+               * presences, and add them in priv->initial_members_aggregator.
+               * When we get the last presence, emit the signal. The last
+               * presence is ourselves. */
+              if (handle == mixin->self_handle)
+                {
+                  /* Change all presences in only one operation */
+                  tp_group_mixin_change_members ((GObject *) chan, "",
+                      tp_handle_set_peek (priv->initial_members_aggregator),
+                      NULL, NULL, NULL, 0, 0);
+                  tp_handle_set_destroy (priv->initial_members_aggregator);
+                  priv->initial_members_aggregator = NULL;
+                }
+            }
 
           if (owner_jid != NULL)
             {
@@ -1807,7 +1842,7 @@ _gabble_muc_channel_handle_subject (GabbleMucChannel *chan,
                                     const gchar *subject,
                                     LmMessage *msg)
 {
-  gboolean error;
+  gboolean is_error;
   GabbleMucChannelPrivate *priv;
   TpIntSet *changed_values, *changed_flags;
   GValue val = { 0, };
@@ -1816,7 +1851,7 @@ _gabble_muc_channel_handle_subject (GabbleMucChannel *chan,
 
   priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (chan);
 
-  error = lm_message_get_sub_type (msg) == LM_MESSAGE_SUB_TYPE_ERROR;
+  is_error = lm_message_get_sub_type (msg) == LM_MESSAGE_SUB_TYPE_ERROR;
 
   if (priv->properties_ctx)
     {
@@ -1824,7 +1859,7 @@ _gabble_muc_channel_handle_subject (GabbleMucChannel *chan,
           ROOM_PROP_SUBJECT);
     }
 
-  if (error)
+  if (is_error)
     {
       LmMessageNode *node;
       const gchar *err_desc = NULL;
@@ -1932,14 +1967,14 @@ _gabble_muc_channel_receive (GabbleMucChannel *chan,
                              const gchar *text,
                              LmMessage *msg)
 {
-  gboolean error;
+  gboolean is_error;
   GabbleMucChannelPrivate *priv;
 
   g_assert (GABBLE_IS_MUC_CHANNEL (chan));
 
   priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (chan);
 
-  error = lm_message_get_sub_type (msg) == LM_MESSAGE_SUB_TYPE_ERROR;
+  is_error = lm_message_get_sub_type (msg) == LM_MESSAGE_SUB_TYPE_ERROR;
 
   if (handle_type == TP_HANDLE_TYPE_ROOM)
     {
