@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-roster-channel.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,7 +29,6 @@
 #include "debug.h"
 #include "gabble-connection.h"
 #include <telepathy-glib/group-mixin.h>
-#include "handles.h"
 #include "roster.h"
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
@@ -36,8 +37,6 @@
 #include <telepathy-glib/svc-channel.h>
 #include "util.h"
 
-#include "gabble-roster-channel.h"
-
 static void channel_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleRosterChannel, gabble_roster_channel,
@@ -95,15 +94,17 @@ gabble_roster_channel_constructor (GType type, guint n_props,
   GabbleRosterChannelPrivate *priv;
   DBusGConnection *bus;
   TpBaseConnection *conn;
-  gboolean valid;
   TpHandle self_handle;
   guint handle_type;
+  TpHandleRepoIface *handle_repo, *contact_repo;
 
   obj = G_OBJECT_CLASS (gabble_roster_channel_parent_class)->
            constructor (type, n_props, props);
   priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (GABBLE_ROSTER_CHANNEL (obj));
   conn = (TpBaseConnection *)priv->conn;
   handle_type = priv->handle_type;
+  handle_repo = tp_base_connection_get_handles (conn, handle_type);
+  contact_repo = tp_base_connection_get_handles (conn, TP_HANDLE_TYPE_CONTACT);
   self_handle = conn->self_handle;
 
   /* register object on the bus */
@@ -114,18 +115,17 @@ gabble_roster_channel_constructor (GType type, guint n_props,
       || handle_type == TP_HANDLE_TYPE_LIST);
 
   /* ref our list handle */
-  valid = tp_handle_ref (conn->handles[handle_type], priv->handle);
-  g_assert (valid);
+  tp_handle_ref (handle_repo, priv->handle);
 
   /* initialize group mixin */
-  tp_group_mixin_init ((TpSvcChannelInterfaceGroup *)obj,
+  tp_group_mixin_init (obj,
       G_STRUCT_OFFSET (GabbleRosterChannel, group),
-      conn->handles[TP_HANDLE_TYPE_CONTACT],
+      contact_repo,
       self_handle);
 
   if (handle_type == TP_HANDLE_TYPE_GROUP)
     {
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
+      tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD |
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
           0);
@@ -137,7 +137,7 @@ gabble_roster_channel_constructor (GType type, guint n_props,
   /* magic contact lists from here down... */
   else if (GABBLE_LIST_HANDLE_PUBLISH == priv->handle)
     {
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
+      tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE |
           TP_CHANNEL_GROUP_FLAG_MESSAGE_ACCEPT |
           TP_CHANNEL_GROUP_FLAG_MESSAGE_REMOVE,
@@ -145,7 +145,7 @@ gabble_roster_channel_constructor (GType type, guint n_props,
     }
   else if (GABBLE_LIST_HANDLE_SUBSCRIBE == priv->handle)
     {
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
+      tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD |
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE |
           TP_CHANNEL_GROUP_FLAG_CAN_RESCIND |
@@ -156,13 +156,13 @@ gabble_roster_channel_constructor (GType type, guint n_props,
     }
   else if (GABBLE_LIST_HANDLE_KNOWN == priv->handle)
     {
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
+      tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
           0);
     }
   else if (GABBLE_LIST_HANDLE_DENY == priv->handle)
     {
-      tp_group_mixin_change_flags ((TpSvcChannelInterfaceGroup *)obj,
+      tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD |
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
           0);
@@ -219,6 +219,10 @@ gabble_roster_channel_set_property (GObject     *object,
     case PROP_OBJECT_PATH:
       g_free (priv->object_path);
       priv->object_path = g_value_dup_string (value);
+    case PROP_CHANNEL_TYPE:
+      /* in telepathy-glib > 0.7.0 this property is writable in the
+       * interface, but not actually meaningfully changeable on this channel,
+       * so we do nothing */
       break;
     case PROP_HANDLE_TYPE:
       priv->handle_type = g_value_get_uint (value);
@@ -238,8 +242,10 @@ gabble_roster_channel_set_property (GObject     *object,
 static void gabble_roster_channel_dispose (GObject *object);
 static void gabble_roster_channel_finalize (GObject *object);
 
-static gboolean _gabble_roster_channel_add_member_cb (TpSvcChannelInterfaceGroup *obj, TpHandle handle, const gchar *message, GError **error);
-static gboolean _gabble_roster_channel_remove_member_cb (TpSvcChannelInterfaceGroup *obj, TpHandle handle, const gchar *message, GError **error);
+static gboolean _gabble_roster_channel_add_member_cb (GObject *obj,
+    TpHandle handle, const gchar *message, GError **error);
+static gboolean _gabble_roster_channel_remove_member_cb (GObject *obj,
+    TpHandle handle, const gchar *message, GError **error);
 
 static void
 gabble_roster_channel_class_init (GabbleRosterChannelClass *gabble_roster_channel_class)
@@ -247,7 +253,8 @@ gabble_roster_channel_class_init (GabbleRosterChannelClass *gabble_roster_channe
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_roster_channel_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_roster_channel_class, sizeof (GabbleRosterChannelPrivate));
+  g_type_class_add_private (gabble_roster_channel_class,
+      sizeof (GabbleRosterChannelPrivate));
 
   object_class->constructor = gabble_roster_channel_constructor;
 
@@ -267,15 +274,18 @@ gabble_roster_channel_class_init (GabbleRosterChannelClass *gabble_roster_channe
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
-  g_object_class_override_property (object_class, PROP_OBJECT_PATH, "object-path");
-  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE, "channel-type");
-  g_object_class_override_property (object_class, PROP_HANDLE_TYPE, "handle-type");
+  g_object_class_override_property (object_class, PROP_OBJECT_PATH,
+      "object-path");
+  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE,
+      "channel-type");
+  g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
+      "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
 
-  tp_group_mixin_class_init ((TpSvcChannelInterfaceGroupClass *)object_class,
-                             G_STRUCT_OFFSET (GabbleRosterChannelClass, group_class),
-                             _gabble_roster_channel_add_member_cb,
-                             _gabble_roster_channel_remove_member_cb);
+  tp_group_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (GabbleRosterChannelClass, group_class),
+      _gabble_roster_channel_add_member_cb,
+      _gabble_roster_channel_remove_member_cb);
 }
 
 void
@@ -304,14 +314,16 @@ gabble_roster_channel_finalize (GObject *object)
   GabbleRosterChannel *self = GABBLE_ROSTER_CHANNEL (object);
   GabbleRosterChannelPrivate *priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (self);
   TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (conn,
+      priv->handle_type);
 
   /* free any data held directly by the object here */
 
   g_free (priv->object_path);
 
-  tp_handle_unref (conn->handles[priv->handle_type], priv->handle);
+  tp_handle_unref (handle_repo, priv->handle);
 
-  tp_group_mixin_finalize ((TpSvcChannelInterfaceGroup *)object);
+  tp_group_mixin_finalize (object);
 
   G_OBJECT_CLASS (gabble_roster_channel_parent_class)->finalize (object);
 }
@@ -333,7 +345,7 @@ _gabble_roster_channel_send_presence (GabbleRosterChannel *chan,
 
   priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (chan);
   conn = (TpBaseConnection *)priv->conn;
-  repo = conn->handles[TP_HANDLE_TYPE_CONTACT];
+  repo = tp_base_connection_get_handles (conn, TP_HANDLE_TYPE_CONTACT);
   contact = tp_handle_inspect (repo, handle);
 
   message = lm_message_new_with_sub_type (contact,
@@ -360,23 +372,28 @@ _gabble_roster_channel_send_presence (GabbleRosterChannel *chan,
  * Called by the group mixin to add one member.
  */
 static gboolean
-_gabble_roster_channel_add_member_cb (TpSvcChannelInterfaceGroup *obj,
+_gabble_roster_channel_add_member_cb (GObject *obj,
                                       TpHandle handle,
                                       const gchar *message,
                                       GError **error)
 {
   GabbleRosterChannelPrivate *priv;
-  TpBaseConnection *conn;
   gboolean ret = FALSE;
+#ifdef ENABLE_DEBUG
+  TpHandleRepoIface *handle_repo, *contact_repo;
+#endif
 
   priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (GABBLE_ROSTER_CHANNEL (obj));
-  conn = (TpBaseConnection *)priv->conn;
+#ifdef ENABLE_DEBUG
+  handle_repo = tp_base_connection_get_handles ((TpBaseConnection *)priv->conn,
+      priv->handle_type);
+  contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+#endif
 
   DEBUG ("called on %s with handle %u (%s) \"%s\"",
-      tp_handle_inspect (conn->handles[priv->handle_type],
-        priv->handle), handle,
-      tp_handle_inspect (conn->handles[TP_HANDLE_TYPE_CONTACT],
-        handle), message);
+      tp_handle_inspect (handle_repo, priv->handle), handle,
+      tp_handle_inspect (contact_repo, handle), message);
 
   if (TP_HANDLE_TYPE_GROUP == priv->handle_type)
     {
@@ -428,22 +445,28 @@ _gabble_roster_channel_add_member_cb (TpSvcChannelInterfaceGroup *obj,
  * Called by the group mixin to remove one member.
  */
 static gboolean
-_gabble_roster_channel_remove_member_cb (TpSvcChannelInterfaceGroup *obj,
+_gabble_roster_channel_remove_member_cb (GObject *obj,
                                          TpHandle handle,
                                          const gchar *message,
                                          GError **error)
 {
   GabbleRosterChannelPrivate *priv;
+#ifdef ENABLE_DEBUG
   TpBaseConnection *conn;
+  TpHandleRepoIface *handle_repo, *contact_repo;
+#endif
   gboolean ret = FALSE;
 
   priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (GABBLE_ROSTER_CHANNEL (obj));
+#ifdef ENABLE_DEBUG
   conn = (TpBaseConnection *)priv->conn;
+  handle_repo = tp_base_connection_get_handles (conn, priv->handle_type);
+  contact_repo = tp_base_connection_get_handles (conn, TP_HANDLE_TYPE_CONTACT);
+#endif
 
-  DEBUG ("called on %s with handle %u (%s) \"%s\"", tp_handle_inspect (
-         conn->handles[priv->handle_type], priv->handle), handle,
-      tp_handle_inspect (conn->handles[TP_HANDLE_TYPE_CONTACT],
-        handle), message);
+  DEBUG ("called on %s with handle %u (%s) \"%s\"",
+      tp_handle_inspect (handle_repo, priv->handle), handle,
+      tp_handle_inspect (contact_repo, handle), message);
 
   if (TP_HANDLE_TYPE_GROUP == priv->handle_type)
     {
@@ -494,8 +517,8 @@ _gabble_roster_channel_remove_member_cb (TpSvcChannelInterfaceGroup *obj,
   else if (GABBLE_LIST_HANDLE_DENY == priv->handle)
     {
       /* unblock contact */
-      ret = gabble_roster_handle_set_blocked (priv->conn->roster, handle, FALSE,
-          error);
+      ret = gabble_roster_handle_set_blocked (priv->conn->roster, handle,
+          FALSE, error);
     }
   else
     {
@@ -608,7 +631,7 @@ gabble_roster_channel_get_interfaces (TpSvcChannel *self,
 
 
 static void
-channel_iface_init(gpointer g_iface, gpointer iface_data)
+channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
