@@ -27,6 +27,7 @@
 #include <telepathy-glib/run.h>
 #include "debug.h"
 #include "gabble-connection-manager.h"
+#include "gabble.h"
 
 static TpBaseConnectionManager *
 construct_cm (void)
@@ -57,27 +58,19 @@ stamp_log (const gchar *log_domain,
 #endif
 
 int
-main (int argc,
-      char **argv)
+gabble_main (int argc,
+             char **argv)
 {
   gabble_debug_set_log_file_from_env ();
 
 #ifdef ENABLE_DEBUG
   gabble_debug_set_flags_from_env ();
 
-  if (g_getenv ("GABBLE_TIMING"))
+  if (g_getenv ("GABBLE_TIMING") != NULL)
     g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, stamp_log, NULL);
 
-  if (g_getenv ("GABBLE_PERSIST"))
-    {
-#ifdef HAVE_TP_DEBUG_SET_FLAGS
-      /* tp-glib >= 0.6.1: persist is no longer a flag in quite the same way */
-      tp_debug_set_persistent (TRUE);
-#else
-      /* tp-glib < 0.6.1: persist is a flag, of sorts */
-      tp_debug_set_flags_from_string ("persist");
-#endif
-    }
+  if (g_getenv ("GABBLE_PERSIST") != NULL)
+    tp_debug_set_persistent (TRUE);
 #endif
 
   return tp_run_connection_manager ("telepathy-gabble", VERSION,
