@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "im-factory.h"
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <stdlib.h>
@@ -36,17 +38,16 @@
 #include "disco.h"
 #include "gabble-connection.h"
 #include "gabble-im-channel.h"
-#include "handles.h"
 #include <telepathy-glib/interfaces.h>
 #include "text-mixin.h"
 #include <telepathy-glib/channel-factory-iface.h>
 
-#include "im-factory.h"
-
-static void gabble_im_factory_iface_init (gpointer g_iface, gpointer iface_data);
+static void gabble_im_factory_iface_init (gpointer g_iface,
+    gpointer iface_data);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleImFactory, gabble_im_factory, G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE, gabble_im_factory_iface_init));
+    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
+      gabble_im_factory_iface_init));
 
 /* properties */
 enum
@@ -65,9 +66,12 @@ struct _GabbleImFactoryPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_IM_FACTORY_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_IM_FACTORY, GabbleImFactoryPrivate))
+#define GABBLE_IM_FACTORY_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_IM_FACTORY,\
+                                GabbleImFactoryPrivate))
 
-static GObject *gabble_im_factory_constructor (GType type, guint n_props, GObjectConstructParam *props);
+static GObject *gabble_im_factory_constructor (GType type, guint n_props,
+    GObjectConstructParam *props);
 
 static void
 gabble_im_factory_init (GabbleImFactory *fac)
@@ -161,7 +165,8 @@ gabble_im_factory_class_init (GabbleImFactoryClass *gabble_im_factory_class)
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_im_factory_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_im_factory_class, sizeof (GabbleImFactoryPrivate));
+  g_type_class_add_private (gabble_im_factory_class,
+      sizeof (GabbleImFactoryPrivate));
 
   object_class->constructor = gabble_im_factory_constructor;
   object_class->dispose = gabble_im_factory_dispose;
@@ -200,8 +205,9 @@ im_factory_message_cb (LmMessageHandler *handler,
   GabbleImFactory *fac = GABBLE_IM_FACTORY (user_data);
   GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
   TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
-
-  const gchar *from, *body, *body_offset;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+  const gchar *from, *body;
   time_t stamp;
   TpChannelTextMessageType msgtype;
   TpHandle handle;
@@ -209,33 +215,38 @@ im_factory_message_cb (LmMessageHandler *handler,
   gint state;
   TpChannelTextSendError send_error;
 
-  if (!gabble_text_mixin_parse_incoming_message (message, &from, &stamp, &msgtype, &body, &body_offset, &state, &send_error))
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+  if (!gabble_text_mixin_parse_incoming_message (message, &from, &stamp,
+        &msgtype, &body, &state, &send_error))
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
   if (body == NULL && state == -1)
     {
-      NODE_DEBUG (message->node, "got a message without a body field, ignoring");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
-  handle = gabble_handle_for_contact (
-      conn->handles[TP_HANDLE_TYPE_CONTACT], from, FALSE);
+  handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
   if (handle == 0)
     {
       NODE_DEBUG (message->node, "ignoring message node from malformed jid");
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
-  DEBUG ("message from %s (handle %u), msgtype %d, body:\n%s",
-         from, handle, msgtype, body_offset);
-
   chan = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
 
   if (chan == NULL)
     {
-      if (send_error != TP_CHANNEL_SEND_NO_ERROR)
+      if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
         {
           DEBUG ("ignoring message error; no sending channel");
+          tp_handle_unref (contact_repo, handle);
+          return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+        }
+
+      if (body == NULL)
+        {
+          /* don't create a new channel if all we have is a chat state */
+          DEBUG ("ignoring message without body; no existing channel");
+          tp_handle_unref (contact_repo, handle);
           return LM_HANDLER_RESULT_REMOVE_MESSAGE;
         }
 
@@ -244,21 +255,37 @@ im_factory_message_cb (LmMessageHandler *handler,
       chan = new_im_channel (fac, handle);
     }
 
-  if (send_error != TP_CHANNEL_SEND_NO_ERROR)
+  g_assert (chan != NULL);
+
+  /* now the channel is referencing the handle, so if we unref it, that's
+   * not a problem */
+  tp_handle_unref (contact_repo, handle);
+
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
+      if (body == NULL)
+        {
+          DEBUG ("ignoring error sending chat state to %s (handle %u)", from,
+              handle);
+          return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+        }
+
+      DEBUG ("got error sending to %s (handle %u), msgtype %u, body:\n%s",
+         from, handle, msgtype, body);
+
       tp_svc_channel_type_text_emit_send_error ((TpSvcChannelTypeText *)chan,
-          send_error, stamp, msgtype, body_offset);
+          send_error, stamp, msgtype, body);
+
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
   if (state != -1)
     _gabble_im_channel_state_receive (chan, state);
 
-  if (body != NULL && _gabble_im_channel_receive (chan, msgtype, handle, from,
-                                  stamp, body_offset))
-    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+  if (body != NULL)
+    _gabble_im_channel_receive (chan, msgtype, handle, from, stamp, body);
 
-  return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
 /**
@@ -315,7 +342,8 @@ new_im_channel (GabbleImFactory *fac, TpHandle handle)
 
   g_hash_table_insert (priv->channels, GINT_TO_POINTER (handle), chan);
 
-  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan, NULL);
+  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan,
+      NULL);
 
   g_free (object_path);
 
@@ -372,8 +400,8 @@ gabble_im_factory_iface_disconnected (TpChannelFactoryIface *iface)
 
   g_assert (priv->message_cb != NULL);
 
-  lm_connection_unregister_message_handler (priv->conn->lmconn, priv->message_cb,
-                                            LM_MESSAGE_TYPE_MESSAGE);
+  lm_connection_unregister_message_handler (priv->conn->lmconn,
+      priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
   lm_message_handler_unref (priv->message_cb);
   priv->message_cb = NULL;
 }
@@ -394,7 +422,9 @@ _foreach_slave (gpointer key, gpointer value, gpointer user_data)
 }
 
 static void
-gabble_im_factory_iface_foreach (TpChannelFactoryIface *iface, TpChannelFunc foreach, gpointer user_data)
+gabble_im_factory_iface_foreach (TpChannelFactoryIface *iface,
+                                 TpChannelFunc foreach,
+                                 gpointer user_data)
 {
   GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
   GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
@@ -417,7 +447,8 @@ gabble_im_factory_iface_request (TpChannelFactoryIface *iface,
 {
   GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
   GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleIMChannel *chan;
   TpChannelFactoryRequestStatus status;
 
@@ -427,8 +458,7 @@ gabble_im_factory_iface_request (TpChannelFactoryIface *iface,
   if (handle_type != TP_HANDLE_TYPE_CONTACT)
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
 
-  if (!tp_handle_is_valid (conn->handles[TP_HANDLE_TYPE_CONTACT],
-        handle, error))
+  if (!tp_handle_is_valid (contact_repo, handle, error))
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_ERROR;
 
   chan = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
