@@ -84,9 +84,6 @@ struct _GabbleMediaFactoryPrivate
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MEDIA_FACTORY, \
                                 GabbleMediaFactoryPrivate))
 
-static GObject *gabble_media_factory_constructor (GType type, guint n_props,
-    GObjectConstructParam *props);
-
 static void
 gabble_media_factory_init (GabbleMediaFactory *fac)
 {
@@ -105,20 +102,6 @@ gabble_media_factory_init (GabbleMediaFactory *fac)
                                                g_free, NULL);
 }
 
-static GObject *
-gabble_media_factory_constructor (GType type, guint n_props,
-                               GObjectConstructParam *props)
-{
-  GObject *obj;
-  GabbleMediaFactoryPrivate *priv;
-
-  obj = G_OBJECT_CLASS (gabble_media_factory_parent_class)->
-           constructor (type, n_props, props);
-  priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (obj);
-
-  return obj;
-}
-
 
 static void
 gabble_media_factory_dispose (GObject *object)
@@ -199,7 +182,6 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
   g_type_class_add_private (gabble_media_factory_class,
       sizeof (GabbleMediaFactoryPrivate));
 
-  object_class->constructor = gabble_media_factory_constructor;
   object_class->dispose = gabble_media_factory_dispose;
 
   object_class->get_property = gabble_media_factory_get_property;
@@ -576,7 +558,7 @@ jingle_info_send_request (GabbleMediaFactory *fac)
  * Called by loudmouth when we get an incoming <iq>. This handler
  * is concerned only with Jingle info queries.
  */
-LmHandlerResult
+static LmHandlerResult
 jingle_info_iq_callback (LmMessageHandler *handler,
                          LmConnection *lmconn,
                          LmMessage *message,
@@ -691,10 +673,10 @@ gabble_media_factory_iface_close_all (TpChannelFactoryIface *iface)
   if (priv->channels)
     {
       GPtrArray *tmp = priv->channels;
-      priv->channels = NULL;
-
       guint i;
 
+      priv->channels = NULL;
+
       for (i = 0; i < tmp->len; i++)
         {
           GabbleMediaChannel *chan = g_ptr_array_index (tmp, i);
