@@ -22,6 +22,9 @@
 
 #include <glib-object.h>
 
+#include "bytestream-iface.h"
+#include "gabble-types.h"
+
 G_BEGIN_DECLS
 
 typedef struct _GabbleMucFactory GabbleMucFactory;
@@ -35,21 +38,31 @@ struct _GabbleMucFactory {
   GObject parent;
 };
 
-GType gabble_muc_factory_get_type(void);
+GType gabble_muc_factory_get_type (void);
 
 /* TYPE MACROS */
 #define GABBLE_TYPE_MUC_FACTORY \
-  (gabble_muc_factory_get_type())
+  (gabble_muc_factory_get_type ())
 #define GABBLE_MUC_FACTORY(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj), GABBLE_TYPE_MUC_FACTORY, GabbleMucFactory))
+  (G_TYPE_CHECK_INSTANCE_CAST((obj), GABBLE_TYPE_MUC_FACTORY, \
+                              GabbleMucFactory))
 #define GABBLE_MUC_FACTORY_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass), GABBLE_TYPE_MUC_FACTORY, GabbleMucFactoryClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass), GABBLE_TYPE_MUC_FACTORY, \
+                           GabbleMucFactoryClass))
 #define GABBLE_IS_MUC_FACTORY(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj), GABBLE_TYPE_MUC_FACTORY))
 #define GABBLE_IS_MUC_FACTORY_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass), GABBLE_TYPE_MUC_FACTORY))
 #define GABBLE_MUC_FACTORY_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_MUC_FACTORY, GabbleMucFactoryClass))
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_MUC_FACTORY, \
+                              GabbleMucFactoryClass))
+
+GabbleMucChannel *gabble_muc_factory_find_text_channel (GabbleMucFactory *self,
+    TpHandle handle);
+
+void gabble_muc_factory_handle_si_stream_request (GabbleMucFactory *self,
+    GabbleBytestreamIface *bytestream, TpHandle room_handle,
+    const gchar *stream_id, LmMessage *msg);
 
 G_END_DECLS
 
