@@ -423,42 +423,6 @@ gabble_presence_cache_set_property (GObject     *object,
   }
 }
 
-#if 0
-static gboolean
-_presence_node_has_google_voice (LmMessageNode *pres_node)
-{
-  LmMessageNode *node;
-  const gchar *cap_ext;
-  gchar **features, **tmp;
-  gboolean found = FALSE;
-
-  node = lm_message_node_get_child_with_namespace (pres_node, "c", NS_CAPS);
-
-  if (node == NULL);
-    return FALSE;
-
-  cap_ext = lm_message_node_get_attribute (node, "ext");
-
-  if (cap_ext == NULL);
-    return FALSE;
-
-  features = g_strsplit (cap_ext, " ", 0);
-
-  for (tmp = features; *tmp; tmp++)
-    {
-      if (!tp_strdiff (tmp, "voice-v1"))
-        {
-          found = TRUE;
-          break;
-        }
-    }
-
-  g_strfreev (features);
-
-  return found;
-}
-#endif
-
 static void
 gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
                                          TpConnectionStatus status,
@@ -601,19 +565,20 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
 
   if (NULL == x_node)
     {
-#if 0
-      if (handle == priv->conn->parent.self_handle)
-        {
-          /* One of my other resources does not support XEP-0153. As per that
-           * XEP, I MUST stop advertising the image hash, at least until all
-           * instances of non-conforming resources have gone offline.
-           * However, we're going to ignore this requirement and hope that
-           * non-conforming clients won't alter the <PHOTO>, which should
-           * in practice be true.
-           */
-          presence->avatar_sha1 = NULL;
-        }
-#endif
+      /* If (handle == priv->conn->parent.self_handle), then this means
+       * that one of our other resources does not support XEP-0153. According
+       * to that XEP, we MUST now stop advertising the image hash, at least
+       * until all instances of non-conforming resources have gone offline, by
+       * setting presence->avatar_sha1 to NULL.
+       *
+       * However, this would mean that logging in (e.g.) with an old version
+       * of Gabble would disable avatars in this newer version, which is
+       * quite a silly failure mode. As a result, we ignore this
+       * requirement and hope that non-conforming clients won't alter the
+       * <PHOTO>, which should in practice be true.
+       *
+       * If handle != self_handle, then in any case we want to ignore this
+       * message for vCard purposes. */
       return;
     }
 
@@ -631,22 +596,16 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
       g_free (presence->avatar_sha1);
       presence->avatar_sha1 = g_strdup (sha1);
 
-#if 0
-      if (handle == priv->conn->parent.self_handle)
-        {
-          /* that would be us, then. According to XEP-0153, we MUST
-           * immediately send a presence update with an empty update child
-           * element (no photo node), then re-download our own vCard;
-           * when that arrives, we may start setting the photo node in our
-           * presence again.
-           *
-           * For the moment I'm going to ignore that requirement and
-           * trust that our other resource is getting its sha1 right!
-           */
-          /* TODO: I don't trust anyone to get XMPP right, so let's do
-           * this. :D */
-        }
-#endif
+      /* FIXME: according to XEP-0153,
+       * if (handle == priv->conn->parent.self_handle), then we MUST
+       * immediately send a presence update with an empty update child
+       * element (no photo node), then re-download our own vCard;
+       * when that arrives, we may start setting the photo node in our
+       * presence again.
+       *
+       * At the moment we ignore that requirement and trust that our other
+       * resource is getting its sha1 right - but it's a good policy to not
+       * trust anyone's XMPP implementation :-) */
 
       g_signal_emit (cache, signals[AVATAR_UPDATE], 0, handle);
     }
@@ -735,10 +694,10 @@ _caps_disco_cb (GabbleDisco *disco,
 
           if (!waiter->disco_requested)
             {
-              const gchar *jid;
+              const gchar *j;
 
-              jid = tp_handle_inspect (contact_repo, waiter->handle);
-              full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
+              j = tp_handle_inspect (contact_repo, waiter->handle);
+              full_jid = g_strdup_printf ("%s/%s", j, waiter->resource);
 
               gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid,
                   node, _caps_disco_cb, cache, G_OBJECT(cache), NULL);
@@ -862,10 +821,10 @@ _caps_disco_cb (GabbleDisco *disco,
 
           if (!waiter->disco_requested)
             {
-              const gchar *jid;
+              const gchar *j;
 
-              jid = tp_handle_inspect (contact_repo, waiter->handle);
-              full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
+              j = tp_handle_inspect (contact_repo, waiter->handle);
+              full_jid = g_strdup_printf ("%s/%s", j, waiter->resource);
 
               gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid,
                   node, _caps_disco_cb, cache, G_OBJECT(cache), NULL);
@@ -1318,3 +1277,20 @@ void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
   info->trust = CAPABILITY_BUNDLE_ENOUGH_TRUST;
   info->caps |= new_caps;
 }
+
+void
+gabble_presence_cache_really_remove (
+    GabblePresenceCache *cache,
+    TpHandle handle)
+{
+  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+  const gchar *jid;
+
+  jid = tp_handle_inspect (contact_repo, handle);
+  DEBUG ("forced to discard cached presence for jid %s", jid);
+  g_hash_table_remove (priv->presence, GINT_TO_POINTER (handle));
+  tp_handle_set_remove (priv->presence_handles, handle);
+}
+
