@@ -38,6 +38,8 @@
 #include "namespaces.h"
 #include "util.h"
 
+#include "presence-cache.h"
+
 #define GOOGLE_ROSTER_VERSION "2"
 
 /* Properties */
@@ -127,7 +129,7 @@ G_DEFINE_TYPE_WITH_CODE (GabbleRoster, gabble_roster, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
       gabble_roster_factory_iface_init));
 
-#define GABBLE_ROSTER_GET_PRIVATE(o)     ((GabbleRosterPrivate*) ((o)->priv));
+#define GABBLE_ROSTER_GET_PRIVATE(o)     ((GabbleRosterPrivate*) ((o)->priv))
 
 static void
 gabble_roster_class_init (GabbleRosterClass *gabble_roster_class)
@@ -511,34 +513,40 @@ static GabbleRosterChannel *_gabble_roster_get_channel (GabbleRoster *,
 
 typedef struct
 {
+  TpHandleRepoIface *contact_repo;
+  TpHandleRepoIface *group_repo;
+  /* TpHandle borrowed from GroupMembershipUpdate => GroupMembershipUpdate */
   GHashTable *group_mem_updates;
+  /* borrowed from the GabbleRosterItem */
   guint contact_handle;
 } GroupsUpdateContext;
 
 typedef struct
 {
-  TpIntSet *contacts_added;
-  TpIntSet *contacts_removed;
-#ifdef ENABLE_DEBUG
+  TpHandleRepoIface *group_repo;
+  TpHandleSet *contacts_added;
+  TpHandleSet *contacts_removed;
+  /* referenced */
   guint group_handle;
-#endif
 } GroupMembershipUpdate;
 
 static GroupMembershipUpdate *
-group_mem_update_ensure (GroupsUpdateContext *ctx, TpHandle group_handle)
+group_mem_update_ensure (GroupsUpdateContext *ctx,
+                         TpHandle group_handle)
 {
   GroupMembershipUpdate *update = g_hash_table_lookup (ctx->group_mem_updates,
       GUINT_TO_POINTER (group_handle));
 
-  if (update) return update;
+  if (update != NULL)
+    return update;
 
   DEBUG ("Creating new hash table entry for group#%u", group_handle);
   update = g_slice_new0 (GroupMembershipUpdate);
-#ifdef ENABLE_DEBUG
+  update->group_repo = ctx->group_repo;
+  tp_handle_ref (update->group_repo, group_handle);
   update->group_handle = group_handle;
-#endif
-  update->contacts_added = tp_intset_new ();
-  update->contacts_removed = tp_intset_new ();
+  update->contacts_added = tp_handle_set_new (ctx->contact_repo);
+  update->contacts_removed = tp_handle_set_new (ctx->contact_repo);
   g_hash_table_insert (ctx->group_mem_updates,
                        GUINT_TO_POINTER (group_handle),
                        update);
@@ -553,7 +561,7 @@ _update_add_to_group (guint group_handle, gpointer user_data)
 
   DEBUG ("- contact#%u added to group#%u", ctx->contact_handle,
          group_handle);
-  tp_intset_add (update->contacts_added, ctx->contact_handle);
+  tp_handle_set_add (update->contacts_added, ctx->contact_handle);
 }
 
 static void
@@ -564,7 +572,7 @@ _update_remove_from_group (guint group_handle, gpointer user_data)
 
   DEBUG ("- contact#%u removed from group#%u", ctx->contact_handle,
          group_handle);
-  tp_intset_add (update->contacts_removed, ctx->contact_handle);
+  tp_handle_set_add (update->contacts_removed, ctx->contact_handle);
 }
 
 static GabbleRosterItem *
@@ -575,17 +583,21 @@ _gabble_roster_item_update (GabbleRoster *roster,
                             gboolean google_roster_mode)
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
   const gchar *ask, *name;
-  TpIntSet *old_groups, *new_groups, *added_to, *removed_from, *removed_from2;
+  TpIntSet *new_groups, *added_to, *removed_from, *removed_from2;
   TpHandleSet *new_groups_handle_set;
-  GroupsUpdateContext ctx = { group_updates, contact_handle };
+  GroupsUpdateContext ctx = { NULL, NULL, group_updates,
+      contact_handle };
+
+  ctx.contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  ctx.group_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
 
   g_assert (roster != NULL);
   g_assert (GABBLE_IS_ROSTER (roster));
-  g_assert (tp_handle_is_valid (contact_repo, contact_handle, NULL));
+  g_assert (tp_handle_is_valid (ctx.contact_repo, contact_handle, NULL));
   g_assert (node != NULL);
 
   item = _gabble_roster_item_get (roster, contact_handle);
@@ -635,12 +647,12 @@ _gabble_roster_item_update (GabbleRoster *roster,
           contact_handle);
     }
 
-  old_groups = tp_handle_set_peek (item->groups);    /* borrowed */
   new_groups_handle_set = _parse_item_groups (node,
       (TpBaseConnection *)priv->conn);
   new_groups = tp_handle_set_peek (new_groups_handle_set);
 
-  removed_from = tp_intset_difference (old_groups, new_groups);
+  removed_from = tp_intset_difference (tp_handle_set_peek (item->groups),
+      new_groups);
   added_to = tp_handle_set_update (item->groups, new_groups);
   removed_from2 = tp_handle_set_difference_update (item->groups, removed_from);
 
@@ -987,17 +999,20 @@ _gabble_roster_received (GabbleRoster *roster)
 static void
 _group_mem_update_destroy (GroupMembershipUpdate *update)
 {
-  tp_intset_destroy (update->contacts_added);
-  tp_intset_destroy (update->contacts_removed);
+  tp_handle_set_destroy (update->contacts_added);
+  tp_handle_set_destroy (update->contacts_removed);
+  tp_handle_unref (update->group_repo, update->group_handle);
   g_slice_free (GroupMembershipUpdate, update);
 }
 
 static gboolean
-_update_group (gpointer key, gpointer value, gpointer user_data)
+_update_group (gpointer key,
+               gpointer value,
+               gpointer user_data)
 {
   guint group_handle = GPOINTER_TO_UINT (key);
   GabbleRoster *roster = GABBLE_ROSTER (user_data);
-  GroupMembershipUpdate *update = (GroupMembershipUpdate *)value;
+  GroupMembershipUpdate *update = value;
   GabbleRosterChannel *group_channel = _gabble_roster_get_channel (
       roster, TP_HANDLE_TYPE_GROUP, group_handle, NULL);
   TpIntSet *empty = tp_intset_new ();
@@ -1008,8 +1023,9 @@ _update_group (gpointer key, gpointer value, gpointer user_data)
 
   DEBUG ("Updating group channel %u now message has been received",
       group_handle);
-  tp_group_mixin_change_members ((GObject *)group_channel,
-      "", update->contacts_added, update->contacts_removed, empty, empty,
+  tp_group_mixin_change_members ((GObject *) group_channel,
+      "", tp_handle_set_peek (update->contacts_added),
+      tp_handle_set_peek (update->contacts_removed), empty, empty,
       0, 0);
 
   tp_intset_destroy (empty);
@@ -1150,6 +1166,14 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
               continue;
             }
 
+          if (strchr (jid, '/') != NULL)
+            {
+              /* Avoid fd.o #12791 */
+              NODE_DEBUG (item_node,
+                  "item node has resource in jid, skipping");
+              continue;
+            }
+
           handle = tp_handle_ensure (contact_repo, jid, NULL, NULL);
           if (handle == 0)
             {
@@ -1340,6 +1364,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       g_hash_table_destroy (group_update_table);
       tp_handle_set_destroy (referenced_handles);
       break;
+
     default:
        NODE_DEBUG (iq_node, "unhandled roster IQ");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
@@ -2046,6 +2071,9 @@ gabble_roster_handle_set_blocked (GabbleRoster *roster,
 
   lm_message_unref (message);
 
+  if (blocked)
+      gabble_presence_cache_really_remove (priv->conn->presence_cache, handle);
+
   return ret;
 }
 
@@ -2326,7 +2354,6 @@ gabble_roster_handle_remove_from_group (GabbleRoster *roster,
   GabbleRosterItem *item;
   LmMessage *message;
   gboolean ret, was_in_group;
-  const gchar *name;
 
   g_return_val_if_fail (roster != NULL, FALSE);
   g_return_val_if_fail (GABBLE_IS_ROSTER (roster), FALSE);
@@ -2362,8 +2389,6 @@ gabble_roster_handle_remove_from_group (GabbleRoster *roster,
       item->unsent_edits = item_edit_new ();
     }
 
-  name = tp_handle_inspect (group_repo, group);
-
   /* temporarily remove the handle from the set (taking a reference),
    * make the message, and put it back afterwards
    */
