@@ -30,11 +30,11 @@ G_BEGIN_DECLS
 
 typedef struct _GabbleRosterClass GabbleRosterClass;
 
-GType gabble_roster_get_type(void);
+GType gabble_roster_get_type (void);
 
 /* TYPE MACROS */
 #define GABBLE_TYPE_ROSTER \
-  (gabble_roster_get_type())
+  (gabble_roster_get_type ())
 #define GABBLE_ROSTER(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj), GABBLE_TYPE_ROSTER, GabbleRoster))
 #define GABBLE_ROSTER_CLASS(klass) \
@@ -68,10 +68,13 @@ typedef enum
 
 GabbleRoster *gabble_roster_new (GabbleConnection *);
 
-GabbleRosterSubscription gabble_roster_handle_get_subscription (GabbleRoster *, TpHandle);
-gboolean gabble_roster_handle_set_blocked (GabbleRoster *, TpHandle, gboolean, GError **);
+GabbleRosterSubscription gabble_roster_handle_get_subscription (GabbleRoster *,
+    TpHandle);
+gboolean gabble_roster_handle_set_blocked (GabbleRoster *, TpHandle, gboolean,
+    GError **);
 const gchar *gabble_roster_handle_get_name (GabbleRoster *, TpHandle);
-gboolean gabble_roster_handle_set_name (GabbleRoster *, TpHandle, const gchar *, GError **);
+gboolean gabble_roster_handle_set_name (GabbleRoster *, TpHandle,
+    const gchar *, GError **);
 gboolean gabble_roster_handle_remove (GabbleRoster *, TpHandle, GError **);
 gboolean gabble_roster_handle_add (GabbleRoster *, TpHandle, GError **);
 gboolean gabble_roster_handle_has_entry (GabbleRoster *, TpHandle);
