@@ -1,6 +1,6 @@
 /*
  * tube-iface.c - Source for GabbleTube interface
- * Copyright (C) 2007 Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@ gboolean
 gabble_tube_iface_accept (GabbleTubeIface *self,
                           GError **error)
 {
-  gboolean (*virtual_method)(GabbleTubeIface *, GError **error) =
+  gboolean (*virtual_method)(GabbleTubeIface *, GError **) =
     GABBLE_TUBE_IFACE_GET_CLASS (self)->accept;
   g_assert (virtual_method != NULL);
   return virtual_method (self, error);
