@@ -392,7 +392,7 @@ tube_opened_cb (GabbleTubeIface *tube,
       TP_TUBE_STATE_OPEN);
 }
 
-GabbleTubeIface *
+static GabbleTubeIface *
 create_new_tube (GabbleTubesChannel *self,
                  TpTubeType type,
                  TpHandle initiator,
@@ -698,21 +698,24 @@ gabble_tubes_channel_presence_updated (GabbleTubesChannel *self,
 
       stream_id = lm_message_node_get_attribute (tube_node, "stream-id");
 
-      extract_tube_information (self, tube_node, NULL,
-          NULL, NULL, NULL, &tube_id);
+      if (!extract_tube_information (self, tube_node, NULL,
+          NULL, NULL, NULL, &tube_id))
+        {
+          DEBUG ("Bad tube ID, skipping to next child of <tubes>");
+          continue;
+        }
+
       tube = g_hash_table_lookup (priv->tubes, GUINT_TO_POINTER (tube_id));
 
       if (tube == NULL)
         {
           /* We don't know yet this tube */
           const gchar *service;
-          TpTubeType type;
           TpHandle initiator_handle;
           GHashTable *parameters;
-          guint tube_id;
 
           if (extract_tube_information (self, tube_node, &type,
-                &initiator_handle, &service, &parameters, &tube_id))
+                &initiator_handle, &service, &parameters, NULL))
             {
               switch (type)
                 {
@@ -1523,7 +1526,7 @@ tube_msg_offered (GabbleTubesChannel *self,
       parameters, NULL, tube_id, NULL);
 }
 
-void
+static void
 tube_msg_close (GabbleTubesChannel *self,
                 LmMessage *msg)
 {
@@ -1660,11 +1663,10 @@ gabble_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
 
   g_free (stream_id);
 #else
-  GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+  GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
       "D-Bus tube support not built" };
 
-  dbus_g_method_return_error (context, &error);
-  return;
+  dbus_g_method_return_error (context, &e);
 #endif
 }
 
@@ -1746,8 +1748,6 @@ gabble_tubes_channel_offer_stream_tube (TpSvcChannelTypeTubes *iface,
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
       /* Stream initiation */
-      GError *error = NULL;
-
       if (!send_new_stream_tube_msg (self, tube, stream_id, &error))
         {
           gabble_tube_iface_close (tube);
@@ -1796,9 +1796,9 @@ gabble_tubes_channel_accept_d_bus_tube (TpSvcChannelTypeTubes *iface,
   tube = g_hash_table_lookup (priv->tubes, GUINT_TO_POINTER (id));
   if (tube == NULL)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "Unknown tube" };
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "Unknown tube" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
@@ -1809,25 +1809,26 @@ gabble_tubes_channel_accept_d_bus_tube (TpSvcChannelTypeTubes *iface,
 
   if (type != TP_TUBE_TYPE_DBUS)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not a D-Bus tube" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
   if (state != TP_TUBE_STATE_LOCAL_PENDING)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the local pending state" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
   if (!gabble_tube_iface_accept (tube, &error))
     {
       dbus_g_method_return_error (context, error);
+      g_error_free (error);
       return;
     }
 
@@ -1842,11 +1843,10 @@ gabble_tubes_channel_accept_d_bus_tube (TpSvcChannelTypeTubes *iface,
   tp_svc_channel_type_tubes_return_from_accept_d_bus_tube (context, addr);
   g_free (addr);
 #else
-  GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+  GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
       "D-Bus tube support not built" };
 
-  dbus_g_method_return_error (context, &error);
-  return;
+  dbus_g_method_return_error (context, &e);
 #endif
 }
 
@@ -1879,9 +1879,9 @@ gabble_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
   tube = g_hash_table_lookup (priv->tubes, GUINT_TO_POINTER (id));
   if (tube == NULL)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "Unknown tube" };
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "Unknown tube" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
@@ -1889,27 +1889,20 @@ gabble_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
       address_type != TP_SOCKET_ADDRESS_TYPE_IPV4 &&
       address_type != TP_SOCKET_ADDRESS_TYPE_IPV6)
     {
-      GError *error = NULL;
-
       error = g_error_new (TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
           "Address type %d not implemented", address_type);
 
       dbus_g_method_return_error (context, error);
-
       g_error_free (error);
       return;
     }
 
   if (access_control != TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
     {
-      GError *error = NULL;
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          "Unix sockets only support localhost control access" };
 
-      error = g_error_new (TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
-          "Unix sockets only support localhost control access");
-
-      dbus_g_method_return_error (context, error);
-
-      g_error_free (error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
@@ -1920,19 +1913,19 @@ gabble_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
 
   if (type != TP_TUBE_TYPE_STREAM)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not a stream tube" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
   if (state != TP_TUBE_STATE_LOCAL_PENDING)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the local pending state" };
 
-      dbus_g_method_return_error (context, &error);
+      dbus_g_method_return_error (context, &e);
       return;
     }
 
@@ -1945,6 +1938,7 @@ gabble_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
   if (!gabble_tube_iface_accept (tube, &error))
     {
       dbus_g_method_return_error (context, error);
+      g_error_free (error);
       return;
     }
 
@@ -2043,11 +2037,10 @@ gabble_tubes_channel_get_d_bus_tube_address (TpSvcChannelTypeTubes *iface,
   g_free (addr);
 
 #else /* ! HAVE_DBUS_TUBE */
-
-  GError error = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+  GError e = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
       "D-Bus tube support not built" };
 
-  dbus_g_method_return_error (context, &error);
+  dbus_g_method_return_error (context, &e);
 #endif
 }
 
@@ -2142,11 +2135,10 @@ gabble_tubes_channel_get_d_bus_names (TpSvcChannelTypeTubes *iface,
   g_ptr_array_free (ret, TRUE);
 
 #else /* HAVE_DBUS_TUBE */
-
-  GError error = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+  GError e = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
       "D-Bus tube support not built" };
 
-  dbus_g_method_return_error (context, &error);
+  dbus_g_method_return_error (context, &e);
 #endif
 }
 
@@ -2166,6 +2158,7 @@ gabble_tubes_channel_get_stream_tube_socket_address (TpSvcChannelTypeTubes *ifac
   GabbleTubeIface *tube;
   TpTubeType type;
   TpTubeState state;
+  TpSocketAddressType address_type;
   GValue *address;
 
   tube = g_hash_table_lookup (priv->tubes, GUINT_TO_POINTER (id));
@@ -2201,11 +2194,12 @@ gabble_tubes_channel_get_stream_tube_socket_address (TpSvcChannelTypeTubes *ifac
     }
 
   g_object_get (tube,
+      "address-type", &address_type,
       "address", &address,
       NULL);
 
   tp_svc_channel_type_tubes_return_from_get_stream_tube_socket_address (
-      context, TP_SOCKET_ADDRESS_TYPE_UNIX, address);
+      context, address_type, address);
 }
 
 /**
@@ -2220,31 +2214,31 @@ gabble_tubes_channel_get_available_stream_tube_types (TpSvcChannelTypeTubes *ifa
 {
   GHashTable *ret;
   GArray *unix_tab, *ipv4_tab, *ipv6_tab;
-  TpSocketAccessControl access;
+  TpSocketAccessControl access_control;
 
   ret = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   /* Socket_Address_Type_Unix */
   unix_tab = g_array_sized_new (FALSE, FALSE, sizeof (TpSocketAccessControl),
       1);
-  access = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
-  g_array_append_val (unix_tab, access);
+  access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
+  g_array_append_val (unix_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_UNIX),
       unix_tab);
 
   /* Socket_Address_Type_IPv4 */
   ipv4_tab = g_array_sized_new (FALSE, FALSE, sizeof (TpSocketAccessControl),
       1);
-  access = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
-  g_array_append_val (ipv4_tab, access);
+  access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
+  g_array_append_val (ipv4_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_IPV4),
       ipv4_tab);
 
   /* Socket_Address_Type_IPv6 */
   ipv6_tab = g_array_sized_new (FALSE, FALSE, sizeof (TpSocketAccessControl),
       1);
-  access = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
-  g_array_append_val (ipv6_tab, access);
+  access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
+  g_array_append_val (ipv6_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_IPV6),
       ipv6_tab);
 
