@@ -264,9 +264,6 @@ lm_message_node_add_build_va (LmMessageNode *node, guint spec, va_list ap)
     {
       switch (arg)
         {
-        case BUILD_END:
-          goto END;
-
         case BUILD_ATTRIBUTE:
           {
             gchar *key = va_arg (ap, gchar *);
@@ -304,10 +301,10 @@ lm_message_node_add_build_va (LmMessageNode *node, guint spec, va_list ap)
 
         case BUILD_POINTER:
           {
-            LmMessageNode **node = va_arg (ap, LmMessageNode **);
+            LmMessageNode **assign_to = va_arg (ap, LmMessageNode **);
 
-            g_return_if_fail (node != NULL);
-            *node = stack->data;
+            g_return_if_fail (assign_to != NULL);
+            *assign_to = stack->data;
           }
           break;
 
@@ -322,7 +319,6 @@ lm_message_node_add_build_va (LmMessageNode *node, guint spec, va_list ap)
       arg = va_arg (ap, guint);
     }
 
-END:
   g_slist_free (stack);
 }
 
@@ -857,53 +853,3 @@ lm_message_node_add_children_from_properties (LmMessageNode *node,
 
   g_hash_table_foreach (properties, set_child_from_property, &data);
 }
-
-#ifndef HAVE_TP_G_HASH_TABLE_UPDATE
-struct _gabble_g_hash_table_update
-{
-  GHashTable *target;
-  GBoxedCopyFunc key_dup, value_dup;
-};
-
-static void
-_gabble_g_hash_table_update_helper (gpointer key,
-                                    gpointer value,
-                                    gpointer user_data)
-{
-  struct _gabble_g_hash_table_update *data = user_data;
-  gpointer new_key = data->key_dup ? (data->key_dup) (key) : key;
-  gpointer new_value = data->value_dup ? (data->value_dup) (value) : value;
-
-  g_hash_table_replace (data->target, new_key, new_value);
-}
-
-/* To be added to tp-glib util.c with s/gabble/tp/ ? */
-
-/**
- * gabble_g_hash_table_update:
- * @target: The hash table to be updated
- * @source: The hash table to update it with (read-only)
- * @key_dup: function to duplicate a key from @source so it can be be stored
- *           in @target. If NULL, the key is not copied, but is used as-is
- * @value_dup: function to duplicate a value from @source so it can be stored
- *             in @target. If NULL, the value is not copied, but is used as-is
- *
- * Add each item in @source to @target, replacing any existing item with the
- * same key. @key_dup and @value_dup are used to duplicate the items; in
- * principle they could also be used to convert between types.
- */
-void
-gabble_g_hash_table_update (GHashTable *target,
-                            GHashTable *source,
-                            GBoxedCopyFunc key_dup,
-                            GBoxedCopyFunc value_dup)
-{
-  struct _gabble_g_hash_table_update data = { target, key_dup,
-      value_dup };
-
-  g_return_if_fail (target != NULL);
-  g_return_if_fail (source != NULL);
-
-  g_hash_table_foreach (source, _gabble_g_hash_table_update_helper, &data);
-}
-#endif
