@@ -675,8 +675,6 @@ static void
 delete_request (GabbleVCardManagerRequest *request)
 {
   GabbleVCardManager *manager = request->manager;
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
-  TpHandleRepoIface *contact_repo;
 
   DEBUG ("Discarding request %p", request);
 
@@ -689,9 +687,6 @@ delete_request (GabbleVCardManagerRequest *request)
    * dangling reference */
   request->manager = NULL;
 
-  contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
-
   request->entry->pending_requests = g_slist_remove
       (request->entry->pending_requests, request);
   cache_entry_attempt_to_free (request->entry);
@@ -835,7 +830,7 @@ observe_vcard (GabbleConnection *conn,
       g_signal_emit (G_OBJECT (manager), signals[NICKNAME_UPDATE], 0, handle);
 }
 
-GError *
+static GError *
 get_error_from_pipeline_reply (LmMessage *reply_msg, GError *error)
 {
     GError *err = NULL;
@@ -969,7 +964,7 @@ patch_vcard_foreach (gpointer k, gpointer v, gpointer user_data)
         }
       else
         {
-          node = lm_message_node_add_child (vcard_node, key, value);
+          lm_message_node_add_child (vcard_node, key, value);
         }
     }
 }
